import time
import tkinter as tk

from workflow_GUI import theme
from workflow_GUI.messagebox import msg_error
from workflow_GUI.dlg_wait_for_actions import WaitForActionsButton

class ChangeEntityNameIdDialog(theme.Toplevel):

   def __init__(self, master, title, width=800, height=540, close_button=False):
      super().__init__(master)
      self.grab_set()
      self.title(title)

      self.columnconfigure(0, weight=1)
      self.rowconfigure(1, weight=1)
      
      # ********* column 0, 1 *************************

      l = theme.Label(self, text="Entities:")
      l.grid(row=0, column=0, sticky="W", padx=10, pady=(20,10))
      self.entity_var = tk.StringVar(self)
      lb = self.entity_lb = theme.Listbox(self, listvar=self.entity_var, selectmode="browse", exportselection=False, width=35, height=20)
      lb.grid(row=1, column=0, sticky="SWNE", padx=(10,2))
      sr = theme.Scrollbar(self, width=20, command=lb.yview)
      lb["yscrollcommand"] = sr.set
      sr.grid(row=1, column=1, sticky="SN")           
      
      # ********** column 2 *************************
      
      f = theme.Frame(self)
      f.grid(row=1, column=2, sticky="SN")
      
      lb = theme.Label(f, text="ID:")
      lb.grid(row=0, column=0, sticky="W", padx=(20,0), pady=(20,0))
      self.id_var = tk.StringVar(self)
      self.id_e = theme.Entry(f, width=10, textvariable=self.id_var)
      self.id_e.grid(row=0, column=1, sticky="W", padx=(10,0), pady=(20,0))
      
      lb = theme.Label(f, text="Name:")
      lb.grid(row=1, column=0, sticky="W", padx=(20,0), pady=(30,0))
      self.name_var = tk.StringVar(self)
      self.name_e = theme.Entry(f, width=30, textvariable=self.name_var)
      self.name_e.grid(row=1, column=1, sticky="W", padx=(10,30), pady=(30,0))
      
      b = theme.Button(f, text="Set changes", width=15, command=self.on_set_change)
      b.grid(row=3, column=1, pady=(30))

      # ********* row 2 ****************************
      
      b = self.wfa_button = WaitForActionsButton(self)
      b.grid(row=2, column=0, sticky="W", padx=(30,0))
      b = theme.Button(self, text="Close", width=8, command=self.on_close)
      b.grid(row=2, column=2, sticky="E", padx=(0,30), pady=(30,20))
         
      # *********** events **************************
      self.bind("<Return>", self.on_set_change)
      self.bind("<Escape>", self.on_close)
      self.bind("<<ListboxSelect>>", self.on_change_entity)
     
      # ********** initialize ************************
      
      self.set_geometry()
      self.set_entity_lb()


   def set_entity_lb(self, a_id=None):
      self.entity_lb.selection_clear(0, "end")
      thread = self.master.perform_isolated_action(self.title(), self.master.diskmod.get_physicals_names)
      ids_names_dict = thread.result
      n = max(len(str(i)) for i in ids_names_dict)
      p = [(id, *name_dim, ("{:"+str(n)+"n} {:s}  {:n}D").format(id, *name_dim)) for id, name_dim in ids_names_dict.items()]
      self.id_2_str = {id:s for id, _, _, s in p}
      self.str_2_id = {s:id for id, _, _, s in p}
      self.name_2_str = {name:s for _, name, _, s in p}
      self.str_2_name = {s:name for _, name, _, s in p}
      self.str_2_dim = {s:dim for _, _, dim, s in p}      
      self.entity_var.set(self.get_sorted(self.str_2_id))      
      lb = self.entity_lb
      if a_id is not None:
         i = list(self.entity_lb.get(0,"end")).index(self.id_2_str[a_id])
         lb.selection_set(i)
         lb.activate(i)
      else:
         lb.selection_set(0)
         lb.activate(0)         
      lb.focus()
      self.on_change_entity()


   def get_sorted(self, items):
      return sorted(items, key=lambda s: self.str_2_id[s])

             
   def on_change_entity(self, envent=None):
      s = self.entity_lb.get("active")
      self.id_var.set(self.str_2_id[s])
      self.name_var.set(self.str_2_name[s])


   def on_set_change(self, event=None):
      id = self.str_2_id[self.entity_lb.get("active")]
      try:
         new_id = int(self.id_var.get())
         if new_id < 0: int("")
      except ValueError:
         msg_error(self, "New ID must be valid integer value more or equal to zero.")
         self.e_id.focus()
         return

      new_name = self.name_var.get().strip()
      if new_name == "":
         msg_error(self, "The name is empty.")
         self.name_e.focus()
         return

      m = self.master
      f = m.create_action_frame(
         m.gen_action_id(),
         self.title(), 
         self.wfa_button.selected_actions_ids,
         m.diskmod.change_physical_id_name, id, new_id, new_name )
      f.start_with_option_protocoling(
         options=[("Entity ID", id), ("New ID",new_id), ("New name", new_name)]
      )
      if f.thread.wait(): self.set_entity_lb(new_id)
      


   def on_close(self, event=None):
      self.destroy()


   # ************* executing ******************

   def execute_modal(self):
      self.wait_window()


