import tkinter as tk
from dialogs.used_widgets_1 import FilenameFrame

from workflow_GUI import theme
from workflow_GUI.messagebox import msg_error

from .data_select import DataSelectionDialog

class DataDepthZlevelRasterDialog(DataSelectionDialog):
   
   def __init__(self, master, title, depth_mode, is_DEM):
      super().__init__(master, title, selectmode="multiple")
      self.depth_mode = depth_mode
     
      # ************** row 0 ****************************

      F = theme.Frame(self.frame, relief="groove", bd=2)
      F.grid(row=0, column=0, sticky="WE", padx=(10,10))
      l = theme.Label(F, text="Grid parameters")
      l.grid(row=0, column=0, columnspan=4)

      w = lambda text: (
         theme.Label(F, text=text, anchor="w"),
         v:=tk.StringVar(self),
         theme.Entry(F, textvariable=v, width=10) )

      l, self.min_x, self.e_min_x = w("X min.:")
      l.grid(row=1, column=0, sticky="w", padx=(10,0), pady=(5,0))
      self.e_min_x.grid(row=1, column=1, sticky="w", padx=(10,0), pady=(5,0))
      l, self.max_x, self.e_max_x = w("X max.:")
      l.grid(row=1, column=2, sticky="w", padx=(30,0), pady=(5,0))
      self.e_max_x.grid(row=1, column=3, sticky="w", padx=(10,10), pady=(5,0))

      l, self.min_y, self.e_min_y = w("Y min.:")
      l.grid(row=2, column=0, sticky="w", padx=(10,0), pady=(5,0))
      self.e_min_y.grid(row=2, column=1, sticky="w", padx=(10,0), pady=(5,0))
      l, self.max_y, self.e_max_y = w("Y max.:")
      l.grid(row=2, column=2, sticky="w", padx=(30,0), pady=(5,0))
      self.e_max_y.grid(row=2, column=3, sticky="w", padx=(10,0), pady=(5,0))

      self.xy_by_ref = tk.BooleanVar(self)
      ch = theme.Checkbutton(F, text="Use x, y as offset to reference position", variable=self.xy_by_ref)
      ch.grid(row=3, column=0, columnspan=4, pady=(5,0))

      l, self.level, self.e_level = w("Depth:" if depth_mode else "Z level")
      l.grid(row=4, column=0, sticky="w", padx=(10,0), pady=(10,0))
      self.e_level.grid(row=4, column=1, sticky="w", padx=(10,0), pady=(10,0))
      
      l, self.grid_step, self.e_grid_step = w("Grid step:")
      self.grid_step.set("10")
      l.grid(row=5, column=0, sticky="w", padx=(10,0), pady=(10,0))
      self.e_grid_step.grid(row=5, column=1, sticky="w", padx=(10,0), pady=(10,10))      
      self.cell_center = tk.BooleanVar(self, value=False)
      ch = theme.Checkbutton(F, text="Cell centered", variable=self.cell_center)
      ch.grid(row=5, column=2, columnspan=2)

      d = self.depth_by_DEM = tk.BooleanVar(self, value=False)
      if is_DEM:
         rb = theme.Radiobutton(F, text="Depth relative to top boundary entities", variable=d, value=False)
         rb.grid(row=6, column=0,columnspan=4, sticky="W", padx=(10,0), pady=(5,0))
         rb = theme.Radiobutton(F, text="Depth relative to DEM", variable=d, value=True)
         rb.grid(row=7, column=0,columnspan=4, sticky="W", padx=(10,0), pady=(5,0))

      # ************* row 1 ****************************

      F = theme.Frame(self.frame, relief="groove", bd=2)
      F.grid(row=1, column=0, sticky="WE", padx=(10,10), pady=(10,0))
      l = theme.Label(F, text="Interpolation parameters")
      l.grid(row=0, column=0, columnspan=2)

      self.nbs_3_nodes = tk.BooleanVar(self, value=True)
      ch = theme.Checkbutton(F, text="Adjacent elements sharing 3 nodes", variable=self.nbs_3_nodes)
      ch.grid(row=1, column=0, columnspan=2, sticky="W", padx=(10,0), pady=(5,0))
      self.nbs_2_nodes = tk.BooleanVar(self, value=True)
      ch = theme.Checkbutton(F, text="Adjacent elements sharing 2 nodes", variable=self.nbs_2_nodes)
      ch.grid(row=2, column=0, columnspan=2, sticky="W", padx=(10,0), pady=(2,0))
      self.nbs_1_nodes = tk.BooleanVar(self, value=True)
      ch = theme.Checkbutton(F, text="Adjacent elements sharing 1 node", variable=self.nbs_1_nodes)
      ch.grid(row=3, column=0, columnspan=2, sticky="W", padx=(10,0), pady=(2,0))
      l = theme.Label(F, text="IDW power:")
      l.grid(row=4, column=0, sticky="W", padx=(10,0), pady=(10,10))
      self.IDW_power = tk.StringVar(self, "2")
      e = theme.Entry(F, textvariable=self.IDW_power, width=8)
      e.grid(row=4, column=1, sticky="W", padx=(10,0), pady=(10,10))


      # ************* rows 2 ************************

      F = theme.Frame(self.frame)
      F.grid(row=2, column=0, sticky="WE", padx=(10,10), pady=(10,0))
      
      l = theme.Label(F, text="Number format:")
      l.grid(row=0, column=0, sticky="W", padx=(10,0), pady=(10,0))
      self.number_format = tk.StringVar(self, value="g")
      self.e_number_format = theme.Entry(F, width=8, textvariable=self.number_format)
      self.e_number_format.grid(row=0, column=1, sticky="W", padx=(15,0), pady=(10,0))

      l = theme.Label(F, text="No value (only *.ASC format):")
      l.grid(row=1, column=0, sticky="W", padx=(10,0), pady=(10,0))
      self.no_value = tk.StringVar(self, value="-9999")
      self.e_no_value = theme.Entry(F, width=8, textvariable=self.no_value)
      self.e_no_value.grid(row=1, column=1, sticky="W", padx=(15,0), pady=(10,0))
      

      # ************* row 3 *******************************
      
      f = self.file_name = FilenameFrame(self.frame, title_above=True)
      f.grid(row=3, column=0, sticky="WE", padx=(10,10), pady=(10,10))
      f.set_save_dialog(
         title="Save Raster Data",
         filetypes=[("ASC file", ".asc"), ("msh file",".msh")],
         defaultextension=".asc",
         initialdir=".")
      
      # *********** initialize *******************
      
      self.set_geometry()



   def _get_range(self, var1, e1, var2, e2, var_name):
      a1 = var1.get().strip()
      if a1 != "": 
         try: a1 = float(a1)
         except:
            msg_error(self.master, "Minimum value of "+var_name+" range is not valid.")
            e1.focus()
            return
      else: a1 = None
      a2 = var2.get().strip()
      if a2 != "": 
         try: a2 = float(a2)
         except:
            msg_error(self.master, "Maximum value of "+var_name+" range is not valid.")
            e2.focus()
            return
      else: a2 = None
      if a1 is not None and a2 is not None:
         if a1 >= a2:
            msg_error(self.master, "Maximum value of "+var_name+" range must be greate than minimum value.")
            e2.focus()
            return
      return a1, a2
         

   def evaluate_dialog(self):
      if (x := self._get_range(self.min_x, self.e_min_x, self.max_x, self.e_max_x, "X")) is None: return
      self.out_x_min, self.out_x_max = x
      if (y := self._get_range(self.min_y, self.e_min_y, self.max_y, self.e_max_y, "Y")) is None: return
      self.out_y_min, self.out_y_max = y
      self.out_xy_by_ref = self.xy_by_ref.get()
      a =  self.level.get().strip()
      try: a = float(a)
      except:
         msg_error(self.master, "Depth level is not valid" if self.depth_mode else "Z level is not valid.")
         self.e_level.focus()
         return            
      self.out_level = a
      a =  self.grid_step.get().strip()
      try: a = float(a)
      except:
         msg_error(self.master, "Grid step is not valid.")
         self.e_grid_step.focus()
         return            
      if a <= 0:
         msg_error(self.master, "Grid step must be greater than zero.")
         self.e_grid_step.focus()
         return
      self.out_grid_step = a
      self.out_cell_center = self.cell_center.get()
      self.out_depth_by_DEM = self.depth_by_DEM.get()
      a = []
      if self.nbs_1_nodes.get(): a.append(1)
      if self.nbs_2_nodes.get(): a.append(2)
      if self.nbs_3_nodes.get(): a.append(3)
      self.out_nbs_nodes_count = a
      a =  self.IDW_power.get().strip()
      try: a = float(a)
      except:
         msg_error(self.master, "IDW power is not valid.")
         self.e_IDW_power.focus()
         return            
      if a <= 0:
         msg_error(self.master, "IDW power must be greater than zero.")
         self.e_IDW_power.focus()
         return
      self.out_IDW_power = a
      self.out_number_format = self.number_format.get().strip()
      s = self.no_value.get().strip()
      try: self.out_no_value = float(s)
      except:
         msg_error(self, "No value '"+s+"' is not valid real number.")
         self.e_no_value.focus()
         return
      if (s:=self.file_name.get_value()) is None: return
      self.out_file_name = s
      return True


   def set_out_params(self):
      if self.evaluate_dialog() is None: return
      return True



