import tkinter as tk
from dialogs.used_widgets_1 import FilenameFrame

from workflow_GUI import theme
from workflow_GUI.messagebox import msg_error

from .data_select import DataSelectionDialog

class DataPointsDialog(DataSelectionDialog):
   
   def __init__(self, master, title, depth_mode, is_DEM):
      super().__init__(master, title, ok_enter_key_event=False, selectmode="multiple")
      self.depth_mode = depth_mode
     
      # ************** row 0 ****************************

      F = theme.Frame(self.frame, relief="groove", bd=2)
      F.grid(row=0, column=0, sticky="WE", padx=(10,10))
      l = theme.Label(F, text="Points description:")
      l.grid(row=0, column=0)
      t = self.t_points = theme.Text(F, width=30, height=8)
      t.grid(row=1, column=0, padx=(10, 10), pady=(5,0))
      if depth_mode:
         l = theme.Label(F, text="Point per line with format (point name, x, y, depth)")
      else:
         l = theme.Label(F, text="Point per line with format (point name, x, y, z)")
      l.grid(row=2, column=0, padx=(10,10), pady=(5,0))
      
      self.xy_by_ref = tk.BooleanVar(self)
      ch = theme.Checkbutton(F, text="Use x, y as offset to reference position", variable=self.xy_by_ref)
      ch.grid(row=3, column=0, pady=(5,5))

      d = self.depth_by_DEM = tk.BooleanVar(self, value=False)
      if is_DEM:
         rb = theme.Radiobutton(F, text="Depth relative to top boundary entities", variable=d, value=False)
         rb.grid(row=4, column=0, sticky="W", padx=(10,0), pady=(0,0))
         rb = theme.Radiobutton(F, text="Depth relative to DEM", variable=d, value=True)
         rb.grid(row=5, column=0, sticky="W", padx=(10,0), pady=(0,5))
      
         

      # ************* row 1 ****************************

      F = theme.Frame(self.frame, relief="groove", bd=2)
      F.grid(row=1, column=0, sticky="WE", padx=(10,10), pady=(10,0))
      l = theme.Label(F, text="Interpolation parameters")
      l.grid(row=0, column=0, columnspan=2)

      self.nbs_3_nodes = tk.BooleanVar(self, value=True)
      ch = theme.Checkbutton(F, text="Adjacent elements sharing 3 nodes", variable=self.nbs_3_nodes)
      ch.grid(row=1, column=0, columnspan=2, sticky="W", padx=(10,0), pady=(5,0))
      self.nbs_2_nodes = tk.BooleanVar(self, value=True)
      ch = theme.Checkbutton(F, text="Adjacent elements sharing 2 nodes", variable=self.nbs_2_nodes)
      ch.grid(row=2, column=0, columnspan=2, sticky="W", padx=(10,0), pady=(2,0))
      self.nbs_1_nodes = tk.BooleanVar(self, value=True)
      ch = theme.Checkbutton(F, text="Adjacent elements sharing 1 node", variable=self.nbs_1_nodes)
      ch.grid(row=3, column=0, columnspan=2, sticky="W", padx=(10,0), pady=(2,0))
      l = theme.Label(F, text="IDW power:")
      l.grid(row=4, column=0, sticky="W", padx=(10,0), pady=(10,10))
      self.IDW_power = tk.StringVar(self, "2")
      e = self.e_IDW_power = theme.Entry(F, textvariable=self.IDW_power, width=8)
      e.grid(row=4, column=1, sticky="W", padx=(10,0), pady=(10,10))


      # ************* rows 2 ************************

      F = theme.Frame(self.frame, relief="groove", bd=2)
      F.grid(row=2, column=0, sticky="WE", padx=(10,10), pady=(10,0))
      
      l = theme.Label(F, text="CSV format")
      l.grid(row=0, column=0, columnspan=4)
      l = theme.Label(F, text="Column separator:")
      l.grid(row=1, column=0, sticky="W")
      v = self.col_sep = tk.StringVar(self, value=";")
      rb = theme.Radiobutton(F, text=",", variable=v, value=",")
      rb.grid(row=1, column=1, sticky="W")
      rb = theme.Radiobutton(F, text=";", variable=v, value=";")
      rb.grid(row=1, column=2, sticky="W")
      rb = theme.Radiobutton(F, text="Tab", variable=v, value="\t")
      rb.grid(row=1, column=3, sticky="W")
      rb = theme.Radiobutton(F, text="Space", variable=v, value=" ")
      rb.grid(row=1, column=4, sticky="W")

      l = theme.Label(F, text="Decimal separator:")
      l.grid(row=2, column=0, sticky="W")
      v = self.decimal_sep = tk.StringVar(self, value=".")
      rb = theme.Radiobutton(F, text=".", variable=v, value=".")
      rb.grid(row=2, column=1, sticky="W")
      rb = theme.Radiobutton(F, text=",", variable=v, value=",")
      rb.grid(row=2, column=2, sticky="W")
      l = theme.Label(F, text="Num. format:")
      l.grid(row=3, column=0, sticky="W")
      self.num_format = tk.StringVar(self, value="g")
      e = self.e_num_format = theme.Entry(F, width=8, textvariable=self.num_format)
      e.grid(row=3, column=1, columnspan=2, sticky="W", pady=(0,5))
      
      # ************* row 3 *******************************
      
      f = self.file_name = FilenameFrame(self.frame, title_above=True, allow_empty=True)
      f.grid(row=3, column=0, sticky="WE", padx=(10,10), pady=(10,10))
      f.set_save_dialog(
         title="Save Raster Data",
         filetypes=[("CSV file", ".csv")],
         defaultextension=".asc",
         initialdir=".")

      # ************ row 4 *********************
      
      self.to_protocol = tk.BooleanVar(self, value=True)
      ch = theme.Checkbutton(self.frame, text="Write to protocol", variable=self.to_protocol)
      ch.grid(row=4, column=0, sticky="W", padx=(10,0), pady=(10,10))
            
      # *********** initialize *******************
      
      self.set_geometry()



   def evaluate_points(self):
      lines = self.t_points.get("0.0","end").strip().split("\n")      
      if len(lines) == 1 and lines[0] == "":
         msg_error(self, "No points description was set.")
         self.t_points.focus()
         return
      lines = [s.strip().replace(";",",").split(",") for s in lines if len(s.strip()) > 0]
      names = []
      if self.depth_mode: self.out_points_xy, self.out_points_depth = [], []
      else: self.out_points = []   
      for i, line in enumerate(lines):
         def err(m): 
            msg_error(self, "Point line {:n}:\n{:s}".format(i+1, m))
            self.t_points.focus()
         if len(line) != 4:
            err("Point description must contain just 4 items (Point name; x; y; z).")
            return
         name, x, y, z_depth = [s.strip() for s in line]
         if name == "":
            err("Name of point is empty.");
            return
         if name in {p[0] for p in names}:
            err("Name of point is already used.")
            return
         names.append(name)
         try: x = float(x)
         except:
            err("Invalid x coordinate")
            return
         try: y = float(y)
         except:
            err("Invalid y coordinate")
            return
         if self.depth_mode:
            try: depth = float(z_depth)
            except:
               err("Invalid depth")
               return
            if depth < 0:
               err("Depth must be greater than or equal to zero.")
               return               
            self.out_points_xy.append((x, y))
            self.out_points_depth.append(depth)
         else:
            try: z = float(z_depth)
            except:
               err("Invalid z coordinate")
               return
            self.out_points.append((x, y, z))
      self.out_points_names = names      
      self.out_xy_by_ref = self.xy_by_ref.get()
      if self.depth_mode:
         self.out_depth_by_DEM = self.depth_by_DEM.get()
      return True


   def evaluate_others(self):         
      a = []
      if self.nbs_1_nodes.get(): a.append(1)
      if self.nbs_2_nodes.get(): a.append(2)
      if self.nbs_3_nodes.get(): a.append(3)
      self.out_nbs_nodes_count = a
      a =  self.IDW_power.get().strip()
      try: a = float(a)
      except:
         msg_error(self.master, "IDW power is not valid.")
         self.e_IDW_power.focus()
         return            
      if a <= 0:
         msg_error(self.master, "IDW power must be greater than zero.")
         self.e_IDW_power.focus()
         return
      self.out_IDW_power = a      
      
      self.out_col_sep = self.col_sep.get()
      self.out_dec_comma = self.decimal_sep.get() == ","
      self.out_num_format = self.num_format.get().strip()
      f = self.out_file_name = self.file_name.get_value()
      p = self.out_write_to_protocol = self.to_protocol.get()
      if f is None and not p:
         msg_error(self, "File name or writing to protocol must be set.")
         return
      return True


   def set_out_params(self):
      if self.evaluate_points() is None : return False
      if self.evaluate_others() is None: return False
      return True

