import tkinter as tk

from workflow_GUI import theme
from workflow_GUI.dlg_wait_for_actions import WaitForActionsButton
from workflow_GUI.messagebox import msg_error
from .used_widgets_1 import FilenameFrame


class DataSelectionDialog(theme.Toplevel):
   
   def __init__(self, master, title, list_width=60, list_height=20, selectmode="multiple", allow_no_selection=False, wfa_button=True, close_button=False, ok_enter_key_event=True):
      super().__init__(master)
      self.last_grab = self.grab_current()
      self.grab_set()
      self.title(title)
      self.allow_no_selection = allow_no_selection

      self.columnconfigure(0, weight=1)
      self.rowconfigure(1, weight=1)


      # ********* column 0, 1 *************************

      l = theme.Label(self, text="Element data (Quantity, Time, Step, Source file):")
      l.grid(row=0, column=0, sticky="W", padx=10, pady=(20,10))
      lb = self.data_lb = theme.Listbox(self, exportselection=False, selectmode=selectmode, width=list_width, height=list_height)      
      lb.grid(row=1, column=0, sticky="SWNE", padx=(10,2))
      sr = theme.Scrollbar(self, width=20, command=lb.yview)
      lb["yscrollcommand"] = sr.set
      sr.grid(row=1, column=1, sticky="SN", padx=(0,10))      
      

      # *********** sorting **********************

      f = theme.Frame(self)
      f.grid(row=2, column=0, sticky="SNWE")
      l = theme.Label(f, text="Sort by:")
      l.grid(row=0, column=0, sticky="W", padx=(10,0), pady=(10,0))
      cb = self.sort_by_cb = theme.Combobox(f, exportselection=False)
      cb.grid(row=0, column=1, sticky="w", padx=(10,0))
      self.data_sorting = [
         ("File, quantity, time", lambda r: (r[0], r[1], r[3]) ),
         ("File, time, quantity", lambda r: (r[0], r[3], r[1]) ),
         ("Quantity, time, file", lambda r: (r[1], r[3], r[0]) ),
         ("Time, quantity, file", lambda r: (r[3], r[1], r[0]) ) ]
      cb["values"] = [s[0] for s in self.data_sorting]
      cb["state"] = "readonly"
      cb.bind("<<ComboboxSelected>>", self.on_set_list_after_sorting)


      # ********* column 2 (additional frame) *************
      
      f = self.frame = theme.Frame(self)
      f.grid(row=1, column=2, sticky="SN")
      

      # ********* row 3 ****************************

      f = theme.Frame(self)
      f.grid(row=3, column=0, columnspan=3, sticky="EW")
      f.columnconfigure(0, weight=1)      
      if wfa_button:
         b = self.wfa_button = WaitForActionsButton(f, dialog=self)
         b.grid(row=0, column=0, sticky="W", padx=(30,0))
      if not close_button:
         b = theme.Button(f, text="Ok", width=8, command=self.on_ok)
         b.grid(row=0, column=1, pady=(30,20))
         b = theme.Button(f, text="Cancel", width=8, command=self.on_cancel)
         b.grid(row=0, column=2, padx=(30,20), pady=(30,30))
      else:
         b = theme.Button(f, text="Close", width=8, command=self.on_cancel)
         b.grid(row=0, column=1, padx=(30,20), pady=(30,30))
      
         
      # *********** events **************************
         
      self.bind("<Escape>", self.on_cancel)
      if not close_button and ok_enter_key_event:
         self.bind("<Return>", self.on_ok)
     
      # ********** initialize ************************
      
      if self.__class__.__base__ is theme.Toplevel: self.set_geometry()

      self.data_lb.focus()
      self.btn_OK = False



   def set_inputs(self, data_refs, selected_refs):
      q = max(len(r[1]) for r in data_refs)
      t = max(len(r[2]) for r in data_refs)
      refs = list(data_refs)
      str_refs = [("{:"+str(q)+"s}  {:"+str(t)+"s}  {:2n}    {:s}").format(*r[1:], r[0]) for r in data_refs]
      self.str_2_refs = {s:r for s, r in zip(str_refs, refs)}
      self.refs_2_str = {r:s for s, r in zip(str_refs, refs)}
      self.sort_by_cb.current(0)      
      self.set_data_lb(selected_refs)
  

   def set_data_lb(self, selected_refs):
      lb = self.data_lb
      if selected_refs is None:
         r = self.get_selected_data_refs()
         if len(r) > 0: selected_refs = r
      i = self.sort_by_cb.current()
      lb.delete(0, "end")      
      refs = sorted(self.refs_2_str, key=self.data_sorting[i][1])
      lb.insert(0, *[self.refs_2_str[r] for r in refs])
      if selected_refs is not None:
         for i, r in enumerate(refs):
            if r in selected_refs: lb.selection_set(i)
      self.data_lb.activate(0)


   def on_set_list_after_sorting(self, event=None):
      self.set_data_lb(None)
      self.do_after_sorting()

   def do_after_sorting(self): pass


   def get_selected_data_refs(self):
      lb = self.data_lb
      return [self.str_2_refs[lb.get(i)] for i in lb.curselection()]


   def destroy(self):
      if self.last_grab is not None: self.last_grab.grab_set()
      super().destroy()


   def on_cancel(self, event=None):
      self.destroy()


   def set_out_params(self):
      return True


   def on_ok(self, event=None):
      refs = self.get_selected_data_refs()
      if not self.allow_no_selection:
         if len(refs) == 0:
            msg_error(self, "No data selected.")
            self.data_lb.focus()
            return
      self.out_selected_refs = refs
      self.out_selected_strings = [self.data_lb.get(i) for i in self.data_lb.curselection()]
      if "wfa_button" in self.__dict__:
         self.out_wfa_ids = self.wfa_button.selected_actions_ids
      if not self.set_out_params(): return
      self.btn_OK = True
      self.destroy()

      


   # ************* executing ******************

   def execute_modal(self):
      self.wait_window()
      return self.btn_OK
