import tkinter as tk

from workflow_GUI import theme
from workflow_GUI.messagebox import msg_error
from workflow_GUI.dlg_wait_for_actions import WaitForActionsButton
from .entity_select import EntitySelectionDialog
from .data_select import DataSelectionDialog
from .used_widgets_1 import FilenameFrame, ListboxScrollbar, RangeFrame


class DataStatisticsDialog(theme.Toplevel):

   def __init__(self, master, title, is_DEM):
      super().__init__(master)
      self.grab_set()
      self.title(title)      

      self.rowconfigure(1, weight=1)
      self.columnconfigure(0, weight=1)
      self.columnconfigure(2, weight=1)
    
      # ************ column 0, 1 **********************

      l = theme.Label(self, text="Entities:")
      l.grid(row=0, column=0, sticky="W", padx=(15,0), pady=(15,0))
      lb = self.lb_entities = theme.Listbox(self, selectmode="browse", exportselection=False, height=15)
      lb.grid(row=1, column=0, sticky="WESN", padx=(15,0), pady=(10,0))
      sr = ListboxScrollbar(lb)
      sr.grid(row=1, column=1, sticky="SN", padx=(2,0), pady=(10,0))           
      self.b_entities = theme.Button(self, text="Add / Remove", width=15, command=self.on_select_entities)
      self.b_entities.grid(row=2, column=0, pady=(10,0))
      
      # *************** column 2, 3 **********************

      l = theme.Label(self, text="Element data (Quantity, Time, Step, Source file):")
      l.grid(row=0, column=2, sticky="W", padx=(15,0), pady=(15,0))
      lb = self.lb_data = theme.Listbox(self, selectmode="browse", exportselection=False, height=15)
      lb.grid(row=1, column=2, sticky="WESN", padx=(15,0), pady=(10,0))
      sr = ListboxScrollbar(lb)
      sr.grid(row=1, column=3, sticky="SN", padx=(2,15), pady=(10,0))                 
      self.b_data = theme.Button(self, text="Add / Remove", width=15, command=self.on_select_data)
      self.b_data.grid(row=2, column=2, pady=(10,0))

      # ********** column 4 ***********************
      
      F = theme.Frame(self)
      F.grid(row=0, column=4, rowspan=5, sticky="SNWE", padx=(10, 10), pady=(10,10))
      

      
      # ********** quantiles *****************************

      Q = theme.Frame(F, relief="groove", bd=2)
      Q.grid(row=0, column=0, sticky="WE")
      self.Q_is = tk.BooleanVar(self, True)
      ch = theme.Checkbutton(Q, text="Quantiles", variable=self.Q_is)
      ch.grid(row=0, column=0)      
      l = theme.Label(Q, text="Parameters:", anchor="w")
      l.grid(row=1, column=0, sticky="W", padx=(10,0), pady=(10,0))            
      cb = self.Q_cb = theme.Combobox(Q, width=22, state="readonly", exportselection=False)
      cb["values"] = ["Quantiles count", "Quntiles list [%]"]
      cb.grid(row=2, column=0, sticky="W", padx=(10,0), pady=(5,0))
      cb.current(0)
      t = self.Q_text = theme.Text(Q, width=20, height=5)
      t.grid(row=3, column=0, padx=(10,10), pady=(5,5))

      # ********** histogram *****************************

      H = theme.Frame(F, relief="groove", bd=2)
      H.grid(row=1, column=0, sticky="WE", pady=(3,0))
      self.H_is = tk.BooleanVar(self, True)
      ch = theme.Checkbutton(H, text="Histogram", variable=self.H_is)
      ch.grid(row=0, column=0)      
      l = theme.Label(H, text="Parameters:", anchor="w")
      l.grid(row=1, column=0, sticky="W", padx=(10,0), pady=(10,0))            
      cb = self.H_cb = theme.Combobox(H, width=22, state="readonly", exportselection=False)
      cb["values"] = ["Edges count", "Edges list"]
      cb.grid(row=2, column=0, sticky="W", padx=(10,0), pady=(5,0))
      cb.current(0)
      t = self.H_text = theme.Text(H, width=20, height=5)
      t.grid(row=3, column=0, padx=(10,10), pady=(5,0))
      self.H_PDF = tk.BooleanVar(self, value=False)
      ch = theme.Checkbutton(H, text="calculate PDF", variable=self.H_PDF)
      ch.grid(row=4, column=0, sticky="W",padx=(10,10), pady=(0,0))

      # ******* statistics type ********************
      
      W = theme.Frame(F, relief="groove", bd=2)
      W.grid(row=2, column=0, sticky="WE", pady=(3,0))
      l = theme.Label(W, text="Data weights:")
      l.grid(row=0, column=0, sticky="W", padx=(10,0))
      w = self.data_weights = tk.StringVar(self, value="no")
      rb = theme.Radiobutton(W, text="No weights", variable=w, value="no")
      rb.grid(row=1, column=0, sticky="W", padx=(10,0))
      rb = theme.Radiobutton(W, text="Element volumes", variable=w, value="volume")
      rb.grid(row=2, column=0, sticky="W", padx=(10,0))
      rb = theme.Radiobutton(W, text="Element surfaces (2D only)", variable=w, value="area")
      rb.grid(row=3, column=0, sticky="W", padx=(10,0), pady=(0,5))

      
      # *********** row 3 ************************

      R = theme.Frame(self)
      R.grid(row=3, column=0, columnspan=4, padx=(15,0), pady=(15,0))      
      
      # ********** CSV format *********************

      F = theme.Frame(R, relief="groove", bd=2)
      F.grid(row=0, column=0, rowspan=2)
      l = theme.Label(F, text="CSV format")
      l.grid(row=0, column=0, columnspan=2)
      l = theme.Label(F, text="Column separator:")
      l.grid(row=1, column=0, columnspan=2, sticky="W")
      v = self.col_sep = tk.StringVar(self, value=";")
      rb = theme.Radiobutton(F, text=",", variable=v, value=",")
      rb.grid(row=2, column=0, sticky="W")
      rb = theme.Radiobutton(F, text=";", variable=v, value=";")
      rb.grid(row=2, column=1, sticky="W")
      rb = theme.Radiobutton(F, text="Tab", variable=v, value="\t")
      rb.grid(row=3, column=0, sticky="W")
      rb = theme.Radiobutton(F, text="Space", variable=v, value=" ")
      rb.grid(row=3, column=1, sticky="W")
      l = theme.Label(F, text="Decimal separator:")
      l.grid(row=4, column=0, columnspan=2, sticky="W")
      v = self.decimal_sep = tk.StringVar(self, value=".")
      rb = theme.Radiobutton(F, text=".", variable=v, value=".")
      rb.grid(row=5, column=0, sticky="W")
      rb = theme.Radiobutton(F, text=",", variable=v, value=",")
      rb.grid(row=5, column=1, sticky="W")
      l = theme.Label(F, text="Num. format:")
      l.grid(row=6, column=0, sticky="W")
      self.num_format = tk.StringVar(self, value="g")
      e = theme.Entry(F, width=8, textvariable=self.num_format)
      e.grid(row=6, column=1, sticky="W", pady=(0,5))
      
      # ************ ranges **********************

      self.range_frame = RangeFrame(R, is_DEM)
      self.range_frame.grid(row=0, column=1, padx=(5,0))
      
      # ******** outputs additional **************

      self.to_protocol = tk.BooleanVar(self, value=True)
      ch = theme.Checkbutton(R, text="Write results into protocol", variable=self.to_protocol)
      ch.grid(row=1, column=1, sticky="E", padx=(0,15), pady=(15,0))

      # *********** row 4 **************************
      
      R = theme.Frame(self)
      R.grid(row=4, column=0, columnspan=5, sticky="WE", padx=(15,15), pady=(10,0))
      R.columnconfigure(0, weight=1)
      R.columnconfigure(1, weight=1)

      # ********** output file names *****************

      f = self.file_name_rows = FilenameFrame(R, title="CSV file name:\n(data as rows)", allow_empty=True)
      f.grid(row=0, column=0, sticky="WE")
      f.set_save_dialog(
         title="Save data statistics",
         filetypes=[("CSV file", ".csv")],
         defaultextension=".csv",
         initialdir=".")
      f = self.file_name_cols = FilenameFrame(R, title="CSV file name:\n(data as columns)", allow_empty=True)
      f.grid(row=0, column=1, sticky="WE", padx=(10,0))
      f.set_save_dialog(
         title="Save data statistics",
         filetypes=[("CSV file", ".csv")],
         defaultextension=".csv",
         initialdir=".")


      # *********** row 5 ************************

      f = theme.Frame(self)
      f.grid(row=5, column=0, columnspan=5, sticky="EW")
      f.columnconfigure(0, weight=1)      
      b = self.wfa_button = WaitForActionsButton(f, dialog=self)
      b.grid(row=0, column=0, sticky="W", padx=(30,0))
      b = theme.Button(f, text="Calculate", width=8, command=self.on_calculate)
      b.grid(row=0, column=1, pady=(30,20))
      b = theme.Button(f, text="Close and calculate", width=18, command=self.on_close_and_calculate)
      b.grid(row=0, column=2, padx=(30,0))      
      b = theme.Button(f, text="Close", width=8, command=self.on_close)
      b.grid(row=0, column=3, padx=(30,20), pady=(30,30))
      
      # *********** events **************************

      self.bind("<Escape>", self.on_close)
      
      
      # *********** initialize ***********************
      
      self.set_geometry()

      self.Q_text.insert("0.0", "10")
      self.H_text.insert("0.0", "10")
      self.lb_entities.focus()
      self.btn_OK = False


   def set_inputs(self, physicals_names_dict, data_refs):
      self.physicals_names_dict = physicals_names_dict
      self.data_refs = data_refs
      self.sel_physicals_ids = []
      self.lb_entities.insert(0, "****** All ******")
      self.sel_data_refs = []
      self.lb_data.insert(0, "****** All ******")


   def on_select_entities(self, event=None):
      d = EntitySelectionDialog(self, "Entities selection", allow_no_selection=True, wfa_button=False)
      d.set_inputs(self.physicals_names_dict, self.sel_physicals_ids)
      if d.execute_modal():
         a = self.sel_physicals_ids = d.out_selected_ids
         self.lb_entities.delete(0, "end")            
         if len(a) > 0: 
            self.lb_entities.insert(0, *d.out_selected_strings) 
         else:
            self.lb_entities.insert(0, "****** All ******")        
            
   
   def on_select_data(self, event=None):
      d = DataSelectionDialog(self, "Element data selection", selectmode="multiple")
      d.set_inputs(self.data_refs, self.sel_data_refs)
      if d.execute_modal():
         a = self.sel_data_refs = d.out_selected_refs
         self.lb_data.delete(0, "end")
         if len(a) > 0:         
            self.lb_data.insert(0, *d.out_selected_strings)
         else:
            self.lb_data.insert(0, "****** All ******") 
      
   
   def eval_quantiles(self):
      s = self.Q_text.get("0.0", "end").strip()
      if (t:=self.Q_cb.current()) == 0:
         try: n = int(s)
         except ValueError:
            msg_error(self, "Value '{:s}' is not valid count of quantiles".format(s))
            self.Q_text.focus()
            return
         if n <= 0:
            msg_error(self, "Count of quantiles must be greater than zero.".format(s))
            self.Q_text.focus()
            return            
         return n
      elif t == 1:
         p = s.replace(";"," ").replace(","," ").replace("\n"," ").replace("\t"," ").split()
         Qs = []
         for s in p:
            try: q = float(s)
            except ValueError:
               msg_error(self, "Value '{:s}' is not valid quantile.".format(s))
               self.Q_text.focus()
               return
            if (q < 0) or (q > 100):
               msg_error(self, "Quantile '{:g}' is not from interval <0;100>.".format(q))
               self.Q_text.focus()
               return
            Qs.append(q)
         if len(Qs) == 0:
            msg_error(self, "No quantile was set.")
            self.Q_text.focus()
            return
         return Qs
         
   
   def eval_histogram(self):
      s = self.H_text.get("0.0", "end").strip()
      if (t:=self.H_cb.current()) == 0:
         try: n = int(s)
         except ValueError:
            msg_error(self, "Value '{:s}' is not valid count of histogram edges.".format(s))
            self.H_text.focus()
            return
         if n <= 1:
            msg_error(self, "Count of histogram edges must be greater than one.".format(s))
            self.H_text.focus()
            return            
         return n
      elif t == 1:
         p = s.replace(";"," ").replace(","," ").replace("\n"," ").replace("\t"," ").split()
         He = []
         for s in p:
            try: x = float(s)
            except ValueError:
               msg_error(self, "Value '{:s}' is not valid histogram edge.".format(s))
               self.H_text.focus()
               return
            He.append(x)
         if len(He) <= 1:
            msg_error(self, "Count of histogram edges must be greater than one.")
            self.H_text.focus()
            return
         for a, b in zip(He[:-1], He[1:]):
            if a >= b:
               msg_error(self, "Histogram edge values are not sorted.")
               self.H_text.focus()
               return
         return He


   def eval_dialog(self):
      self.out_sel_physicals_ids = p if len(p:=self.sel_physicals_ids) > 0 else None
      if (r := self.range_frame.get_values()) is None: return
      self.out_range = r
      self.out_sel_data_refs = d if len(d:=self.sel_data_refs) > 0 else None
      if self.Q_is.get():
         if (Q:=self.eval_quantiles()) is None: return
         self.out_quantiles = Q
      else: self.out_quantiles = None
      if self.H_is.get():
         if (H:=self.eval_histogram()) is None: return
         self.out_histogram = H
      else: self.out_histogram = None
      self.out_histogram_PDF = self.H_PDF.get()
      self.out_weights_type = t if (t:=self.data_weights.get()) != "no" else None
      if t == "area":
         p, d = self.out_sel_physicals_ids, self.physicals_names_dict
         if p is None: p = list(d)
         if len([True for id in p if d[id][1] != 2]) > 0:
            msg_error(self, "Area weights are given only for 2D entities.")
            return
      self.out_file_name_rows = r = self.file_name_rows .get_value()
      self.out_file_name_cols = c = self.file_name_cols .get_value()
      self.out_to_protocol = p = self.to_protocol.get()
      if r is None and c is None and not p:
         msg_error(self, "No output for calculation was not set.")
         return
      self.out_col_sep = self.col_sep.get()
      self.out_dec_comma = self.decimal_sep.get() == ","
      self.out_num_format = self.num_format.get().strip()
      self.out_wfa_ids = self.wfa_button.selected_actions_ids
      return True
      

   def get_protocol_inputs(self):
      p = self.master.create_protocol_partition_params(self.out_sel_physicals_ids, self.out_range)
      p.extend(self.master.get_protocol_e_data_refs(self.out_sel_data_refs))         
      if type(q:=self.out_quantiles is int):
         p.append(("Quantiles count", q))
      elif type(q) is list:
         p.append(("Quantiles [%]", q))
      if type(h:=self.out_histogram) is int:
         p.append(("Histogram edges count", h))
      elif type(h) is list:
         p.append(("Histogram edges", h))
      if type(h) is not None:
         p.append(("Histogram density", self.out_histogram_PDF))         
      if (w:=self.out_weights_type) is not None:
         p.append(("Weights type", w))
      if (r:=self.out_file_name_rows) is not None:
         p.append(("Table file name (data rows)", r))
      if (c:=self.out_file_name_cols) is not None:
         p.append(("Table file name (data cols)", c))
      if r is not None or c is not None:
         p.append(("Column separator", {" ":"space", "\t":"tabulator", ";":"semicolon", ",":"comma"}[self.out_col_sep]))
         p.append(("Decimal comma", self.out_dec_comma))
      if (s:=self.out_num_format) != "g":         
         p.append(("Number format", '"' + s + '"'))
      if (t:=self.out_to_protocol):
         p.append(("Write to protocol", t))         
      return p


   def on_calculate(self):
      if self.eval_dialog() is None: return
      m, action_name = self.master, self.title()
      thread = m.perform_isolated_action(action_name, m.diskmod.calc_e_data_statistics,
           physicals_ids=self.out_sel_physicals_ids, **self.out_range,
           data_refs=self.out_sel_data_refs, weights_type=self.out_weights_type, quantiles=self.out_quantiles, hist_bins=self.out_histogram, hist_as_pdf=self.out_histogram_PDF,
           file_name_rows=self.out_file_name_rows, file_name_cols=self.out_file_name_cols, include_data_file_name=False, to_protocol=self.out_to_protocol, col_sep=self.out_col_sep, dec_comma=self.out_dec_comma, num_format=self.out_num_format)
      if thread is None:
         msg_error(self, "Calculation terminated with error.")
         return
      if self.out_to_protocol:
         thread.action_id = m.gen_action_id()
         thread.wait_for_actions_ids = self.out_wfa_ids
         m.protocol_perform_action(thread, 
            inputs = self.get_protocol_inputs(),
            results = thread.result )
      
   
   def on_close_and_calculate(self):
      if self.eval_dialog() is None: return
      self.btn_OK = True
      self.destroy()


   def on_close(self, events=None):
      self.destroy()


   # ************* executing ******************

   def execute_modal(self):
      self.wait_window()
      return self.btn_OK




