import tkinter as tk

from workflow_GUI import theme
from workflow_GUI.messagebox import msg_error
from workflow_GUI.dlg_wait_for_actions import WaitForActionsButton

class EntityCrossSectionDialog(theme.Toplevel):

   def __init__(self, master, title, width=800, height=540, close_button=False):
      super().__init__(master)
      self.grab_set()
      self.title(title)

      self.columnconfigure(0, weight=1)
      self.rowconfigure(1, weight=1)
      
      # ********* column 0, 1 *************************

      l = theme.Label(self, text="Entities:")
      l.grid(row=0, column=0, sticky="W", padx=10, pady=(20,10))
      self.entity_var = tk.StringVar(self)
      lb = self.entity_lb = theme.Listbox(self, listvar=self.entity_var, selectmode="browse", exportselection=False, width=35, height=20)
      lb.grid(row=1, column=0, sticky="SWNE", padx=(10,2))
      sr = theme.Scrollbar(self, width=20, command=lb.yview)
      lb["yscrollcommand"] = sr.set
      sr.grid(row=1, column=1, sticky="SN")           
      
      # ********** column 2 *************************
      
      f = theme.Frame(self)
      f.grid(row=1, column=2, sticky="SN")
      
      l = theme.Label(f, text="Default cross Section:")
      l.grid(row=0, column=0, sticky="W", padx=(20,0), pady=(20,0))
      self.default_cross = tk.StringVar(self, "1")
      self.default_cross_e = theme.Entry(f, width=10, textvariable=self.default_cross)
      self.default_cross_e.grid(row=0, column=1, sticky="W", padx=(10,15), pady=(20,0))
      
      l = theme.Label(f, text="Entity cross section:")
      l.grid(row=1, column=0, sticky="W", padx=(20,0), pady=(60,0))
      self.entity_cross = tk.StringVar(self)
      self.entity_cross_e = theme.Entry(f, width=10, textvariable=self.entity_cross)
      self.entity_cross_e.grid(row=1, column=1, sticky="W", padx=(10,20), pady=(60,0))      
      b = theme.Button(f, text="Set", width=5, command=self.on_set_change)
      b.grid(row=2, column=1, pady=(10,0))

      # ********* row 2 ****************************
      
      f = theme.Frame(self)
      f.grid(row=3, column=0, columnspan=3, sticky="EW")
      f.columnconfigure(0, weight=1)           
      b = self.wfa_button = WaitForActionsButton(f, dialog=self)
      b.grid(row=0, column=0, sticky="W", padx=(30,0))      
      b = theme.Button(f, text="Ok", width=8, command=self.on_ok)
      b.grid(row=0, column=1, pady=(30,20))
      b = theme.Button(f, text="Cancel", width=8, command=self.on_cancel)
      b.grid(row=0, column=2, padx=(30,20), pady=(30,30))
         
      # *********** events **************************
      
      self.bind("<Return>", self.on_set_change)
      self.bind("<Escape>", self.on_cancel)
      self.bind("<<ListboxSelect>>", self.on_change_entity)
     
      # ********** initialize ************************
      
      self.set_geometry()
      self.btn_OK = False

      

   def set_inputs(self, physical_names_dict, default_cross):
      self.physicals_names_dict = physical_names_dict
      self.default_cross.set("{:g}".format(default_cross))
      self.set_entity_lb()
      

   def set_entity_lb(self, a_id=None):
      self.entity_lb.selection_clear(0, "end")
      n = max(len(str(i)) for i in self.physicals_names_dict)      
      self.id_2_str = {id: ("{:"+str(n)+"n} {:s}  {:n}D").format(id,*c[:2]) + ("  (default)" if c[2] is None else "  ({:g})".format(c[2])) for id, c in self.physicals_names_dict.items()}
      self.str_2_id = {s:id for id, s in self.id_2_str.items()}                 
      self.entity_var.set(self.get_sorted(self.str_2_id))
      lb = self.entity_lb
      if a_id is not None:
         i = list(lb.get(0,"end")).index(self.id_2_str[a_id])
         lb.selection_set(i)
         lb.activate(i)
      else:
         lb.selection_set(0)
         lb.activate(0)
      self.on_change_entity()
      lb.focus()


   def get_sorted(self, items):
      return sorted(items, key=lambda s: self.str_2_id[s])


   def get_lb_id(self):
      s = self.entity_lb.get("active")
      return self.str_2_id[s]

             
   def on_change_entity(self, envent=None):
      id = self.get_lb_id()
      c = self.physicals_names_dict[id][2]
      if c is not None:
         self.entity_cross.set("{:g}".format(c))
      else:
         self.entity_cross.set("")
      

   def on_set_change(self, event=None):
      id = self.get_lb_id()
      s = self.entity_cross.get().strip()
      if len(s) > 0:
         try: 
            c = float(s)
            if c <= 0: float("")
         except ValueError:
            msg_error(self, "Entity cross section must be valid real value greater than zero.")
            self.entity_cross_e.focus()
            return            
      else:
         c = None
      self.physicals_names_dict[id][2] = c
      self.set_entity_lb(id)
      

   def on_ok(self, event=None):
      try: 
         c = float(self.default_cross.get().strip())
         if c <= 0: float("")
      except ValueError:
         msg_error(self, "Default cross section must be valid real value greater than zero.")
         self.default_cross_e.focus()
         return 
      self.out_default = c
      self.out_entities = {id:v[2] for id, v in self.physicals_names_dict.items() if v[2] is not None}
      self.out_wfa_ids = self.wfa_button.selected_actions_ids      
      self.btn_OK = True
      self.destroy()


   def on_cancel(self, event=None):
      self.destroy()


   # ************* executing ******************

   def execute_modal(self):
      self.wait_window()
      return self.btn_OK


