import re
import tkinter as tk

from workflow_GUI import theme
from workflow_GUI.messagebox import msg_info
from workflow_GUI.dlg_wait_for_actions import WaitForActionsButton

class EntitySelectionDialog(theme.Toplevel):

   def __init__(self, master, title, list_width=35, list_height=24, allow_no_selection=False, wfa_button=True, close_button=False):
      super().__init__(master)
      self.last_grab = self.grab_current()
      self.grab_set()
      self.title(title)
      self.allow_no_selection = allow_no_selection

      self.columnconfigure(0, weight=1)
      self.columnconfigure(3, weight=1)
      self.rowconfigure(1, weight=1)
      
      # ********* column 0, 1 *************************

      l = theme.Label(self, text="Non selected entity:")
      l.grid(row=0, column=0, sticky="W", padx=10, pady=(20,10))
      v = self.non_sel_var = tk.StringVar(self)
      lb = self.non_sel_lb = theme.Listbox(self, listvar=v, selectmode="extended", width=list_width, height=list_height)      
      lb.grid(row=1, column=0, sticky="SWNE", padx=(10,2))
      sr = theme.Scrollbar(self, width=20, command=lb.yview)
      lb["yscrollcommand"] = sr.set
      sr.grid(row=1, column=1, sticky="SN")      
      lb.bind("<Insert>", self.on_add)
      
      # ******** column 2 ************************

      f = theme.Frame(self, width=50)
      f.grid(row=1, column=2, sticky="SNWE")

      b = theme.Button(f, text="Append from list >>", command=self.on_add)
      b.grid(row=0, column=0, sticky="N", padx=20, pady=(0, 20))
      b = theme.Button(f, text="<< Remove from list", command=self.on_delete)
      b.grid(row=1, column=0, sticky="N", padx=20, pady=(0,20))
      
      b = theme.Button(f, text="Append by text >>", command=self.on_text_add)
      b.grid(row=2, column=0, sticky="N", padx=20, pady=(30, 2))
      x = self.ids_text = theme.Text(f, height=5, width=16)
      x.grid(row=3, column=0, sticky="WE", padx=10)
      b = theme.Button(f, text="<< Remove by text", command=self.on_text_delete)
      b.grid(row=4, column=0, sticky="N", padx=20, pady=(2,0))

      b = theme.Button(f, text="<< Remove all", command=self.on_delete)
      b.grid(row=5, column=0, sticky="N", padx=20, pady=(30,0))


      # ******** column 3, 4 ********************

      l = theme.Label(self, text="Selected entity:")
      l.grid(row=0, column=3, sticky="W", padx=10, pady=(20,10))
      v = self.sel_var = tk.StringVar(self)
      lb = self.sel_lb = theme.Listbox(self, listvar=v, selectmode="extended", width=list_width, height=list_height)      
      lb.grid(row=1, column=3, sticky="SNWE", padx=(10,2))
      lb.bind("<Delete>", self.on_delete)
      sr = theme.Scrollbar(self, width=20, command=lb.yview)
      lb["yscrollcommand"] = sr.set
      sr.grid(row=1, column=4, sticky="SN", padx=(0,10))

      # ********** column 5 *************************
      self.frame = theme.Frame(self)
      self.frame.grid(row=0, column=5, rowspan=2, sticky="SN")


      # ********* row 2 ****************************
      
      f = theme.Frame(self)
      f.grid(row=2, column=0, columnspan=6, sticky="EW")
      f.columnconfigure(0, weight=1)      
      if wfa_button:
         b = self.wfa_button = WaitForActionsButton(f, dialog=self)
         b.grid(row=0, column=0, sticky="W", padx=(30,0))
      if not close_button:
         b = theme.Button(f, text="Ok", width=8, command=self.on_ok)
         b.grid(row=0, column=1, pady=(30,20))
         b = theme.Button(f, text="Cancel", width=8, command=self.on_cancel)
         b.grid(row=0, column=2, padx=(30,20), pady=(30,30))
      else:
         b = theme.Button(f, text="Close", width=8, command=self.on_cancel)
         b.grid(row=0, column=1, padx=(30,20), pady=(30,30))
         
      # *********** events **************************         
      self.bind("<Escape>", self.on_cancel)
      if not close_button:
         self.bind("<Return>", self.on_ok)
     
      # ********** initialize ************************
      
      if self.__class__.__base__ is theme.Toplevel: self.set_geometry()

      self.non_sel_lb.focus()
      self.btn_OK = False


   def set_inputs(self, ids_names_dict, selected_ids):
      n = max(len(str(i)) for i in ids_names_dict)
      p = [(id, *name_dim, ("{:"+str(n)+"n} {:s}  {:n}D").format(id, *name_dim)) for id, name_dim in ids_names_dict.items()]
      self.id_2_str = {id:s for id, _, _, s in p}
      self.str_2_id = {s:id for id, _, _, s in p}
      self.name_2_str = {name:s for _, name, _, s in p}
      self.str_2_name = {s:name for _, name, _, s in p}
      self.str_2_dim = {s:dim for _, _, dim, s in p}
      s = {self.id_2_str[id] for id in selected_ids}
      self.non_sel_var.set(self.get_sorted(set(self.str_2_id) - s))
      self.sel_var.set(self.get_sorted(s))
      self.non_sel_lb.activate(0)
      self.non_sel_lb.selection_set(0)


   def get_sorted(self, items):
      return sorted(items, key=lambda s: self.str_2_id[s])


   def get_non_sel_ids(self):
      return [self.str_2_id[s] for s in self.non_sel_lb.get(0, "end")]
   
   def get_sel_ids(self):
      return [self.str_2_id[s] for s in self.sel_lb.get(0, "end")]

   def get_non_sel_names(self):
      return [self.str_2_name[s] for s in self.non_sel_lb.get(0, "end")]
   
   def get_sel_names(self):
      return [self.str_2_name[s] for s in self.sel_lb.get(0, "end")]

   def get_sel_dims(self):
      return {self.str_2_dim[s] for s in self.sel_lb.get(0, "end")}


   def destroy(self):
      if self.last_grab is not None: self.last_grab.grab_set()
      super().destroy()

             
   # **************** buttons *********************

   def on_add(self, event=None):
      nlb, slb = self.non_sel_lb, self.sel_lb
      c = nlb.curselection()
      if len(c) == 0: 
         msg_info(self, "The list of non-selected entities has no selection.")
         return
      for i0 in c:
         i = self.get_sorted((*slb.get(0,"end"), s:=nlb.get(i0))).index(s)
         slb.insert(i if i < slb.size() else "end", s)
      nlb.delete(c[0], c[-1])
      if nlb.size() == c[0]:
         nlb.selection_set(c[0] - 1)
      if nlb.size() > 0:       
         nlb.selection_set(c[0])
      slb.see(i)

     
   def on_delete(self, event=None):
      nlb, slb = self.non_sel_lb, self.sel_lb
      c = slb.curselection()
      if len(c) == 0: 
         msg_info(self, "The list of selected entities has no selection.")
         return
      for i in c:
         i0 = self.get_sorted((*nlb.get(0,"end"), s:=slb.get(i))).index(s)
         nlb.insert(i0 if i0 < nlb.size() else "end", s)
      slb.delete(c[0], c[-1])
      if slb.size() == c[0]:
         slb.selection_set(c[0] - 1)
      if slb.size() > 0:       
         slb.selection_set(c[0])
      nlb.see(i0)


   def eval_ids_text(self):
      ids = set()
      p = self.ids_text.get("0.0", "end").replace(","," ").split()
      for s in p:
         if re.search("^[0-9]+$", s) is not None:
            ids.add(int(s))
         elif re.search("^[0-9]+-[0-9]+$", s) is not None:
            i1, i2 = list(map(int, s.split("-")))
            if i2 < i1: 
               msg_info(self, "First id in value '"+s+"' must be less or equal to first value.")
               return
            ids.update(range(i1,i2+1))
         else:
            msg_info(self, "Value '"+s+"' is not valid id or range of ids.")
            return
      return ids
      

   def on_text_add(self, event=None):
      if (ids := self.eval_ids_text()) is None: return
      s = {self.id_2_str[id] for id in ids}
      self.sel_var.set(self.get_sorted(set(self.sel_lb.get(0,"end")) | s))
      self.non_sel_var.set(self.get_sorted(set(self.non_sel_lb.get(0,"end")) - s))
      

   def on_text_delete(self, event=None):
      if (ids := self.eval_ids_text()) is None: return
      s = {self.id_2_str[id] for id in ids}
      self.non_sel_var.set(self.get_sorted(set(self.non_sel_lb.get(0,"end")) | s))
      self.sel_var.set(self.get_sorted(set(self.sel_lb.get(0,"end")) - s))


   def set_out_params(self):
      return True


   def on_ok(self, event=None):
      if self.focus_get() is self.ids_text: return
      ids = self.get_sel_ids()      
      if not self.allow_no_selection:
         if len(ids) == 0:
            msg_info(self, "No entity was selected.")
            return
      self.out_selected_ids = ids
      self.out_selected_strings = list(self.sel_lb.get(0, "end"))
      if "wfa_button" in self.__dict__:
         self.out_wfa_ids = self.wfa_button.selected_actions_ids
      if not self.set_out_params(): return
      self.btn_OK = True
      self.destroy()


   def on_cancel(self, event=None):
      self.destroy()


   # ************* executing ******************

   def execute_modal(self):
      self.wait_window()
      return self.btn_OK


