import tkinter as tk

from workflow_GUI import theme
from workflow_GUI.messagebox import msg_error
from workflow_GUI.dlg_wait_for_actions import WaitForActionsButton
from .entity_select import EntitySelectionDialog


class FluxBetweenEntitiesDialog(theme.Toplevel):

   def __init__(self, master, title):
      super().__init__(master)
      self.grab_set()
      self.title(title)      

      self.rowconfigure(1, weight=1)
      self.columnconfigure(0, weight=1)
      self.columnconfigure(1, weight=1)
    
      # ************ column 0 **********************

      l = theme.Label(self, text="Main entities:")
      l.grid(row=0, column=0, sticky="W", padx=(15,0), pady=(15,0))
      lb = self.lb_main_entities = theme.Listbox(self, width=35, height=20)
      lb.grid(row=1, column=0, sticky="WESN", padx=(15,10), pady=(10,0))
      self.b_main_entities = theme.Button(self, text="Add / Remove", width=15, command=self.on_select_main_entities)
      self.b_main_entities.grid(row=2, column=0, pady=(10,0))
      
      # *************** column 1 **********************

      l = theme.Label(self, text="Neighbouring entities:")
      l.grid(row=0, column=1, sticky="W", padx=(15,0), pady=(15,0))
      lb = self.lb_nbs_entities = theme.Listbox(self, width=35, height=20)
      lb.grid(row=1, column=1, sticky="WESN", padx=(15,10), pady=(10,0))
      self.b_nbs_entities = theme.Button(self, text="Add / Remove", width=15, command=self.on_select_nbs_entities)
      self.b_nbs_entities.grid(row=2, column=1, pady=(10,0))
      
      # *********** column 2 ***************************
      
      R = theme.Frame(self)
      R.grid(row=1, column=2, sticky="SNEW", padx=20)
      b = theme.Button(R, text="Calculate", width=15, command=self.on_calculate)
      b.grid(row=0, column=0, pady=(15,0))
      self.l_Q_in = theme.Label(R)
      self.l_Q_in.grid(row=1, column=0, sticky="W", pady=(20,0))
      self.l_Q_out = theme.Label(R)
      self.l_Q_out.grid(row=2, column=0, sticky="W", pady=(20,0))
      self.by_vector_var = tk.BooleanVar(self)
      self.ch_vector = theme.Checkbutton(R, text="By vector direction (x, y, z):", variable=self.by_vector_var, command=self.on_by_vector)
      self.ch_vector.grid(row=3, column=0, sticky="W", pady=(30,0))
      self.vector_var = tk.StringVar(self)
      self.vector_var.trace_add("write", self.on_change_vector)
      self.e_vector = theme.Entry(R, width=24, state="disable", textvariable=self.vector_var)
      self.e_vector.grid(row=4, column=0, sticky="W", pady=(10,0))
      self.l_Qv_in = theme.Label(R)
      self.l_Qv_in.grid(row=5, column=0, sticky="W", pady=(20,0))
      self.l_Qv_out = theme.Label(R)
      self.l_Qv_out.grid(row=6, column=0, sticky="W", pady=(20,0))
      self.to_protocol_var = tk.BooleanVar(self, value=True)
      self.ch_protocol = theme.Checkbutton(R, text="Write preview to protocol", variable=self.to_protocol_var)
      self.ch_protocol.grid(row=7, column=0, sticky="W", pady=(30,0))


      # ************* row 3 **************************
      
      f =theme.Frame(self)
      f.grid(row=3, column=0, columnspan=3, sticky="SNWE")
      f.columnconfigure(1, weight=1)
      self.b_wfa = WaitForActionsButton(f,dialog=self)
      self.b_wfa.grid(row=0, column=0, padx=20, pady=(30, 20))
      b = theme.Button(f, text="Close and calculate", command=self.on_OK)
      b.grid(row=0, column=1, sticky="E", padx=10)
      b = theme.Button(f, text="Close", width=10, command=self.on_close)
      b.grid(row=0, column=2, sticky="E", padx=20)
      

      # ************ events ******************************
      self.bind("<Escape>", self.on_close)

      # ************* initialize ******************
      self.set_geometry()
      self.set_Q()
      self.set_Qv()
      self.b_main_entities.focus()
      self.btn_OK = False


   def set_inputs(self, physicals_names_dict):
      self.physicals_names_dict = physicals_names_dict
      self.main_physicals_ids = []
      self.nbs_physicals_ids = []
      self.vector = None


   def set_Q(self, *Q):
      f = self.master.float2str
      self.l_Q_in["text"] = "Flux in: " + (f(Q[0], "flux") if len(Q) else "")
      self.l_Q_out["text"] = "Flux out: " + (f(Q[1], "flux") if len(Q) else "")


   def set_Qv(self, *Qv):
      f = self.master.float2str
      self.l_Qv_in["text"] = "Flux in: " + (f(Qv[0], "flux") if len(Qv) else "")
      self.l_Qv_out["text"] = "Flux out: " + (f(Qv[1], "flux") if len(Qv) else "")


   def on_by_vector(self):
      if self.by_vector_var.get():
         self.e_vector["state"] = "normal"
      else:
         self.vector_var.set("")
         self.e_vector["state"] = "disable"


   def on_change_vector(self, *args):
      self.set_Qv()


   def on_select_main_entities(self):
      d = EntitySelectionDialog(self, "Main entities selection")
      a = {id:v for id, v in self.physicals_names_dict.items() if id not in self.nbs_physicals_ids}  
      d.set_inputs(a, self.main_physicals_ids)
      if d.execute_modal():
         self.main_physicals_ids = d.out_selected_ids
         self.lb_main_entities.delete(0, "end")
         self.lb_main_entities.insert(0, *d.out_selected_strings)
         self.set_Q()
         self.set_Qv()


   def on_select_nbs_entities(self):
      d = EntitySelectionDialog(self, "Main entities selection")
      a = {id:name for id, name in self.physicals_names_dict.items() if id not in self.main_physicals_ids}  
      d.set_inputs(a, self.nbs_physicals_ids)
      if d.execute_modal():
         self.nbs_physicals_ids = d.out_selected_ids
         self.lb_nbs_entities.delete(0, "end")
         self.lb_nbs_entities.insert(0, *d.out_selected_strings)
         self.set_Q()
         self.set_Qv()


   def vector_evaluate(self):
      s = self.vector_var.get().replace(","," ").split()
      if len(s) != 3:
         msg_error(self, "Vector must contain just three separated values.")
         self.e_vector.focus()
         return         
      x = y = z = None
      try:
         x = float(s[0])
         y = float(s[1])
         z = float(s[2])
      except ValueError:
         r = "x" if x is None else ("y" if y is None else "z")
         msg_error(self, "Vector coordinate " + r + " contain invalid floating point value.")
         self.e_vector.focus()
         return
      return [x, y, z]


   def evaluate(self):
      if self.by_vector_var.get():
         vec = self.vector_evaluate()
         if vec is None: return False
      else: vec = None      
      self.out_vector = vec
      self.out_T = 0
      m = self.main_physicals_ids
      if len(m) == 0:
         msg_error(self, "Main entity group is empty.")
         self.b_main_entities.focus()
         return False
      self.out_main_entities_ids = m
      m = self.nbs_physicals_ids
      if len(m) == 0:
         msg_error(self, "Neighbouring entities list is empty.")
         self.b_nbs_entities.focus()
         return False
      self.out_nbs_entities_ids = m
      self.out_wfa_ids = self.b_wfa.selected_actions_ids
      return True

            
   def on_calculate(self):
      if not self.evaluate(): return
      m, action_name = self.master, self.title()
      thread = m.perform_isolated_action(action_name, m.diskmod.calc_e4_flux_over_e2e4_neighbours, self.out_main_entities_ids, self.out_nbs_entities_ids, self.out_T, vector=self.out_vector)
      if thread is None:
         msg_error(self, "Calculation terminated with error.")
         return
      r = thread.result
      self.set_Q(r[0], r[1])
      if len(r) == 4: self.set_Qv(r[2], r[3])
      if self.to_protocol_var.get():
         thread.action_id = m.gen_action_id()
         thread.wait_for_actions_ids = self.out_wfa_ids
         m.protocol_perform_action(thread, 
            inputs = self.get_protocol_inputs(),
            results = self.get_protocol_results(r) )
         

   def get_protocol_inputs(self):
      return [
         ("Main entities IDs", self.out_main_entities_ids),
         ("Neighbours entities IDs", self.out_nbs_entities_ids),
         ("Time", self.out_T),
         ("Vector", self.out_vector if self.out_vector is not None else "")
         ]


   def get_protocol_results(self, Q):
      f = self.master.float2str
      p = [("Q_in",f(Q[0], "flux")), ("Q_out", f(Q[1], "flux"))]
      if len(Q) == 4: 
         p += [("Q_in by vector direction", f(Q[2], "flux")), 
               ("Q_out by vector direction", f(Q[3], "flux")) ]
      return p


   def on_OK(self):
      if not self.evaluate(): return
      self.btn_OK = True
      self.destroy()

   def on_close(self, event=None):
      self.destroy()


   # ************* executing ******************

   def execute_modal(self):
      self.wait_window()
      return self.btn_OK


