import tkinter as tk
import tkinter.filedialog as filedialog

from workflow_GUI import theme
from workflow_GUI.messagebox import msg_error
from workflow_GUI.dlg_wait_for_actions import WaitForActionsButton

class LevelAboveZChangeDialog(theme.Toplevel):

   def __init__(self, master, title):
      super().__init__(master)
      self.grab_set()
      self.title(title)
      self.resizable(False, False)

      f = theme.Frame(self)
      f.grid(row=0, column=0, columnspan=3, sticky="W", padx=(20,20), pady=(30,15))
      lb = theme.Label(f, text="Z Level:")
      lb.grid(row=0, column=0, sticky="W")
      self.z_level_var = tk.StringVar(self)
      self.e_level = theme.Entry(f, textvariable=self.z_level_var, width=10)
      self.e_level.grid(row=0, column=1, padx=20)

      self.write_file_var = tk.BooleanVar(self, value=True)
      ch = theme.Checkbutton(self, text="Write msh file with change differences", variable=self.write_file_var, command=self.on_write_file)
      ch.grid(row=1, column=0, columnspan=3, sticky="w", padx=20, pady=(20,0))
      
      f = theme.Frame(self)
      f.grid(row=2, column=0, columnspan=3, sticky="W", padx=(20, 20), pady=(10,30))
      lb = theme.Label(f, text="File name:")
      lb.grid(row=0, column=0, sticky="W")
      self.file_var = tk.StringVar(self)
      self.file_e = theme.Entry(f, width=60, textvariable=self.file_var)
      self.file_e.grid(row=0, column=1, sticky="W", padx=(20,8))
      self.file_dlg_b = theme.Button(f, text="...", width=3, command=self.on_choose_file_name)
      self.file_dlg_b.grid(row=0, column=2)
            
      self.wfa_button = WaitForActionsButton(self)
      self.wfa_button.grid(row=3, column=0, sticky="W", padx=20, pady=20)
      b_ok = theme.Button(self, text="OK", width=8, command=self.on_ok)
      b_ok.grid(column=1, row=3, padx=(180,0), pady=20)
      b = theme.Button(self, text="Cancel", width=8, command=self.on_cancel)
      b.grid(column=2, row=3, padx=(15,30), pady=20)

      # *********** events **************************
      self.bind("<Escape>", self.on_cancel)
      self.bind("<Return>", self.on_ok)

      # ************ initialize ********************
      
      self.set_geometry()
      
      self.e_level.focus()
      self.btn_OK = False


   def on_write_file(self):
      if self.write_file_var.get():
         self.file_e["state"] = "normal"
         self.file_e.focus()
         self.file_dlg_b["state"] = "normal"
      else:
         self.file_e["state"] = "disable"
         self.file_var.set("")
         self.file_dlg_b["state"] = "disable"
    
          
   def on_choose_file_name(self):
      file_name = filedialog.asksaveasfilename(         
         title="Save Mesh File",
         filetypes=[("gmsh file", ".msh")],
         defaultextension=".msh",
         initialdir=".")
      if file_name is not None: self.file_var.set(file_name)


   def on_ok(self, event=None):
      try:
         v = float(self.z_level_var.get())
      except ValueError:
         msg_error(self, "Z level is not valid real number.")
         self.e_level.focus()
         return
      self.out_z_level = v
      if self.write_file_var.get():
         s = self.file_var.get().strip()
         if s == "":
            msg_error(self, message="File name was not set.")
            return
         self.out_file_name = s
      else: self.out_file_name = None
      self.out_wfa_ids = self.wfa_button.selected_actions_ids
      self.btn_OK = True
      self.destroy()


   def on_cancel(self, event=None):
      self.destroy()


   # ************* executing ******************

   def execute_modal(self):
      self.wait_window()
      return self.btn_OK


