import tkinter as tk
import tkinter.filedialog as filedialog

from workflow_GUI import theme
from workflow_GUI.dlg_wait_for_actions import WaitForActionsButton

class OpenMeshDialog(theme.Toplevel):

   def __init__(self, master, title):
      super().__init__(master)
      self.grab_set()
      self.title(title)

      # ******* input / output ****************
      self.mesh_file = tk.StringVar(master)
      self.physical_names_file = tk.StringVar(master)
      
      self.mesh_file.trace_add("write", self.trace_mesh_file)

      # ************** row 0 ****************************
      l = theme.Label(self, text="Mesh file:")
      l.grid(column=0, row=0, sticky="W", padx=10)
      e = self.e_mesh_file = theme.Entry(self,textvariable=self.mesh_file, width=50)
      e.grid(column=1, row=0, columnspan=3, pady=20)
      b = theme.Button(self, text="...", command=self.on_open_msh, width=4)
      b.grid(column=4, row=0, sticky="W", padx=5)

      # **************** row 1 *********************
      v = self.physical_names_source = tk.StringVar(self)
      v.trace_add("write", self.trace_physical_names_source)
      l = theme.Label(self, text="Physical names source:")
      l.grid(column=0, row=1, sticky="W", padx=10)      
      r = theme.Radiobutton(self, text="Mesh file", variable=self.physical_names_source, value="mesh_file")
      r.grid(column=1, row=1)
      r = theme.Radiobutton(self, text="Extra mesh file", variable=self.physical_names_source, value="extra_mesh_file")
      r.grid(column=2, row=1)
      r = theme.Radiobutton(self, text="No physical names", variable=self.physical_names_source, value="no_physical_names")
      r.grid(column=3, row=1)

      # ************* row 2 *****************************
      l = theme.Label(self, text="Physical names file:")
      l.grid(column=0, row=2, sticky="W", padx=10)
      e = self.e_physical_names = theme.Entry(self, textvariable=self.physical_names_file, width=50)
      e.grid(column=1, row=2, columnspan=3, pady=20)
      b = self.b_physical_names = theme.Button(self, text="...", command=self.on_physical_names, width=4)
      b.grid(column=4, row=2, sticky="W", padx=5)

      # ***************** row 3 *********************
      b = self.wfa_button = WaitForActionsButton(self)
      b.grid(row=3, column=0)
      b = theme.Button(self, text="OK", command=self.on_ok, width=10)
      b.grid(column=2, row=3, sticky="E", pady=30)
      b = theme.Button(self, text="Cancel", command=self.on_cancel, width=10)
      b.grid(column=3, row=3, sticky="E", pady=30)
   
      # ********** events ************************
      self.bind("<Escape>", self.on_cancel)
      self.bind("<Return>", self.on_ok)

      # ********** initialize ********************
      
      self.set_geometry()

      self.btn_OK = False
      self.physical_names_source.set("mesh_file")
      self.e_mesh_file.focus()


   # ************ events ****************************

   def trace_mesh_file(self, v, index, mode):
      if self.physical_names_source.get() == "mesh_file":
         self.physical_names_file.set(self.mesh_file.get())


   def trace_physical_names_source(self, v, index, mode):
      if self.physical_names_source.get() == "mesh_file":
         self.e_physical_names.configure(state="readonly")
         self.physical_names_file.set(self.mesh_file.get())
         self.b_physical_names.configure(state=tk.DISABLED)
      elif self.physical_names_source.get() == "extra_mesh_file":
         self.e_physical_names.configure(state="normal")
         self.b_physical_names.configure(state=tk.NORMAL)
      else:
         self.e_physical_names.configure(state="readonly")
         self.physical_names_file.set("")
         self.b_physical_names.configure(state=tk.DISABLED)

        


   # ************* buttons *************************

   def show_open_dialog(self):
    file_name = filedialog.askopenfilename(
        title="Open Mesh File",
        filetypes=[("gmsh file", ".msh")],
        defaultextension=".msh",
        initialdir=".")
    return file_name


   def on_open_msh(self):
      if (s:=self.show_open_dialog()) is None: return
      self.mesh_file.set(s)
       

   def on_physical_names(self):
      if (s:=self.show_open_dialog()) is None: return
      self.physical_names_file.set(s)
      

   def on_ok(self, event=None):
      self.out_wfa_ids = self.wfa_button.selected_actions_ids
      self.btn_OK = True
      self.destroy()


   def on_cancel(self, event=None):
      self.destroy()



   # ************ execute ***********************

   def execute_modal(self):
      self.wait_window()
      return self.btn_OK

