
import tkinter as tk
import tkinter.filedialog as filedialog

from workflow_GUI import theme
from workflow_GUI.dlg_wait_for_actions import WaitForActionsButton
from workflow_GUI.messagebox import msg_error

from .used_widgets_1 import FilenameFrame

class OpenOutputDataDialog(theme.Toplevel):

   def __init__(self, master, title):
      super().__init__(master)
      self.grab_set()
      self.title(title)

      self.columnconfigure(0, weight=1)
      self.columnconfigure(1, weight=1)
      self.columnconfigure(2, weight=1)
      self.rowconfigure(3, weight=1)

      f = self.file_name = FilenameFrame(self)
      f.grid(row=0, column=0, columnspan=3, sticky="WE", padx=20, pady=(20,0))
      f.set_open_dialog(
         title="Open Mesh File",
         filetypes=[("gmsh file", ".msh")],
         defaultextension=".msh",
         initialdir=".")
      
      # ********** row 1 ***************************

      b = theme.Button(self, text="Get data names / times", width=24, command=self.on_get_names_times)
      b.grid(row=1, column=0, columnspan=3, pady=(10,0))                       
      



      # ************ names **************************

      l = theme.Label(self, text="Names:")
      l.grid(row=2, column=0, sticky="W", padx=(20,0), pady=(10,0))
      self.names_text = theme.Text(self, width=20)
      self.names_text.grid(row=3, column=0, sticky="WESN", padx=(20,0), pady=5)
      self.names_all_var = tk.BooleanVar(self, value=False)
      ch = theme.Checkbutton(self, text="All", variable=self.names_all_var, command=self.on_names_all)
      ch.grid(row=4, column=0, padx=(20,0), pady=5)
      
      # ************ times ****************************
      
      l = theme.Label(self, text="Times:")
      l.grid(row=2, column=1, sticky="W", padx=(20,0), pady=(10,0))
      self.times_text = theme.Text(self, width=20)
      self.times_text.grid(row=3, column=1, sticky="WESN", padx=(20,0), pady=5)
      self.times_all_var = tk.BooleanVar(self, value=False)
      ch = theme.Checkbutton(self, text="All", variable=self.times_all_var, command=self.on_times_all)
      ch.grid(row=4, column=1, padx=(20,0), pady=5)

      # ************ times steps ****************************
      
      l = theme.Label(self, text="Times steps:")
      l.grid(row=2, column=2, sticky="W", padx=(20,0), pady=(10,0))
      self.steps_text = theme.Text(self, width=20)
      self.steps_text.grid(row=3, column=2, sticky="WESN", padx=(20,20), pady=5)
      self.steps_all_var = tk.BooleanVar(self, value=False)
      ch = theme.Checkbutton(self, text="All", variable=self.steps_all_var, command=self.on_steps_all)
      ch.grid(row=4, column=2, padx=(20,0), pady=5)

      # *********** buttons ******************************
      
      f = theme.Frame(self)
      f.grid(row=5, column=0, columnspan=3, sticky="WESN")
      f.columnconfigure(0, weight=1)
      b = self.wfa_button = WaitForActionsButton(f, dialog=self)
      b.grid(row=0, column=0, sticky="W", padx=(20,20), pady=(30,20))
      b = theme.Button(f, text="OK", width=8, command=self.on_OK)
      b.grid(row=0, column=1, padx=(0,30), pady=(30,20))      
      b = theme.Button(f, text="Cancel", width=8, command=self.on_cancel)
      b.grid(row=0, column=2, padx=(0,30), pady=(30,20))

      # ************ events *************************
      self.bind("<Escape>", self.on_cancel)


      # ************* initialize ****************
      
      self.set_geometry()

      self.names_text.focus()
      self.btn_OK = False


   def on_get_names_times(self):
      file_name = self.file_name.get_value()
      if file_name is None: return
      m, action_name = self.master, self.title()
      thread = m.perform_isolated_action(action_name, m.diskmod.load_msh_element_data_references, file_name=file_name)     
      if thread is None: return
      s = "\n".join(sorted({r[1] for r in thread.result}))
      self.names_text.delete("0.0", "end")
      self.names_text.insert("0.0", s)
      s = "\n".join(sorted({r[2] for r in thread.result}, key=lambda s: float(s)))
      self.times_text.delete("0.0", "end")
      self.times_text.insert("0.0", s)
      s = "\n".join([str(i) for i in sorted({r[3] for r in thread.result})])
      self.steps_text.delete("0.0", "end")
      self.steps_text.insert("0.0", s)
      

   def on_names_all(self):
      if self.names_all_var.get():
         self.names_text.delete("0.0", "end")
         self.names_text["state"] = "disable"         
      else:
         self.names_text["state"] = "normal"
         self.names_text.focus()


   def on_times_all(self):
      if self.times_all_var.get():
         self.times_text.delete("0.0", "end")
         self.times_text["state"] = "disable"
      else:
         self.times_text["state"] = "normal"
         self.times_text.focus()


   def on_steps_all(self):
      if self.steps_all_var.get():
         self.steps_text.delete("0.0", "end")
         self.steps_text["state"] = "disable"
      else:
         self.steps_text["state"] = "normal"
         self.steps_text.focus()


   def eval_names_text(self):
      A = self.names_text.get("0.0", "end").strip().split("\n")
      p = [s.strip() for s in A if s.strip() != ""]
      if len(p) == 0:
         msg_error(self, "No name was set.")
         self.names_text.focus()
         return               
      return p


   def eval_times_text(self):
      A = self.times_text.get("0.0", "end").strip().split("\n")
      p = [s.strip() for s in A if s.strip() != ""]      
      if len(p) == 0:
         msg_error(self, "No times was set.")
         self.times_text.focus()
         return      
      for t in p:
         try: float(t)
         except:
            msg_error(self, "Value of time {:s} is not vaild.".format(t))
            self.times_text.focus()
            return
      return p


   def eval_steps_text(self):
      A = self.steps_text.get("0.0", "end").strip().split("\n")
      p = [s.strip() for s in A if s.strip() != ""]      
      if len(p) == 0:
         msg_error(self, "No time steps was set.")
         self.steps_text.focus()
         return            
      for i in p:
         try: int(i)
         except:
            msg_error(self, "Value of time step {:s} is not vaild.".format(i))
            self.times_text.focus()
            return        
      return list(map(int, p))


   def on_OK(self):
      self.out_msh_file = self.file_name.get_value()
      if self.out_msh_file is None: return
      if self.names_all_var.get(): self.out_names = None
      else:
         self.out_names = self.eval_names_text()
         if self.out_names is None: return
      if self.times_all_var.get(): self.out_times = None
      else:
         self.out_times = self.eval_times_text()
         if self.out_times is None: return
      if self.steps_all_var.get(): self.out_steps = None
      else:
         self.out_steps = self.eval_steps_text()
         if self.out_steps is None: return
      self.out_wfa_ids = self.wfa_button.selected_actions_ids
      self.btn_OK = True
      self.destroy()
      

   def on_cancel(self, event=None):
      self.destroy()
      



   # ************ execute ***********************

   def execute_modal(self):
      self.wait_window()
      return self.btn_OK



   