import tkinter as tk
import tkinter.filedialog as filedialog

from workflow_GUI import theme
from workflow_GUI.dlg_wait_for_actions import WaitForActionsButton
from workflow_GUI.messagebox import msg_error

class ReferencePointDialog(theme.Toplevel):

   def __init__(self, master, title):
      super().__init__(master)
      self.grab_set()
      self.title(title)


      # ************** row 0 ****************************
      
      l = theme.Label(self, text="X:")
      l.grid(column=0, row=0, sticky="W", padx=(40,0), pady=(30,0))
      self.x_ref = tk.StringVar(self)
      e = self.e_x_ref = theme.Entry(self, textvariable=self.x_ref, width=20)
      e.grid(column=1, row=0, sticky="W", padx=(10,0), pady=(30,0))      
      b = theme.Button(self, text="Minimum x", command=self.on_min_x)
      b.grid(column=2, row=0, sticky="W", padx=(40,0), pady=(30,0))
      b = theme.Button(self, text="Maximum x", command=self.on_max_x)
      b.grid(column=3, row=0, sticky="W", padx=(15,40), pady=(30,0))

 
      # ************** row 1 ****************************
      
      l = theme.Label(self, text="Y:")
      l.grid(column=0, row=1, sticky="W", padx=(40,0), pady=(20,0))
      self.y_ref = tk.StringVar(self)
      e = self.e_y_ref = theme.Entry(self, textvariable=self.y_ref, width=20)
      e.grid(column=1, row=1, sticky="W", padx=(10,0), pady=(20,15))      
      b = theme.Button(self, text="Minimum y", command=self.on_min_y)
      b.grid(column=2, row=1, sticky="W", padx=(40,0), pady=(20,15))
      b = theme.Button(self, text="Maximum y", command=self.on_max_y)
      b.grid(column=3, row=1, sticky="W", padx=(15,40), pady=(20,15))


      # ********** row 2 *************************
      
      f = theme.Frame(self)
      f.grid(row=2, column=0, columnspan=4, sticky="EW")
      f.columnconfigure(0, weight=1)          
      b = self.wfa_button = WaitForActionsButton(f, dialog=self)
      b.grid(row=0, column=0, sticky="W", padx=(30,0))
      b = theme.Button(f, text="Ok", width=8, command=self.on_ok)
      b.grid(row=0, column=1, pady=(30,20))
      b = theme.Button(f, text="Cancel", width=8, command=self.on_cancel)
      b.grid(row=0, column=2, padx=(30,20), pady=(30,30))
              
      # *********** events **************************
         
      self.bind("<Escape>", self.on_cancel)
      self.bind("<Return>", self.on_ok)

      # ********** initialize ********************
      
      self.set_geometry(resizable=False)
      
      self.e_x_ref.focus()
      self.btn_OK = False
      
              
   def set_inputs(self, x, y, x_min, y_min, x_max, y_max):
      self.x_ref.set("{:g}".format(x))
      self.y_ref.set("{:g}".format(y))
      self.x_min, self.y_min = x_min, y_min
      self.x_max, self.y_max = x_max, y_max



   def on_min_x(self):
      self.x_ref.set("{:g}".format(self.x_min))
   
   def on_min_y(self):
      self.y_ref.set("{:g}".format(self.y_min))

   def on_max_x(self):
      self.x_ref.set("{:g}".format(self.x_max))

   def on_max_y(self):
      self.y_ref.set("{:g}".format(self.y_max))




   def on_ok(self, event=None):
      try: x = float(self.x_ref.get().strip())
      except:
         msg_error(self, "X coordinate is not valid number.")
         self.e_x_ref.focus()
         return
      self.out_x_ref = x
      try: y = float(self.y_ref.get().strip())
      except:
         msg_error(self, "Y coordinate is not valid number.")
         self.e_y_ref.focus()
         return
      self.out_y_ref = y
      self.out_wfa_ids = self.wfa_button.selected_actions_ids
      self.btn_OK = True
      self.destroy()


   def on_cancel(self, event=None):
      self.destroy()



   # ************ execute ***********************

   def execute_modal(self):
      self.wait_window()
      return self.btn_OK

