import tkinter as tk

from workflow_GUI import theme
from workflow_GUI.messagebox import msg_error
from workflow_GUI.dlg_wait_for_actions import WaitForActionsButton
from .entity_select import EntitySelectionDialog
from .data_select import DataSelectionDialog
from .used_widgets_1 import ListboxScrollbar


class SetBoundaryDataDialog(theme.Toplevel):

   def __init__(self, master, title):
      super().__init__(master)
      self.grab_set()
      self.title(title)      

      self.rowconfigure(1, weight=1)
      self.columnconfigure(0, weight=1)
      self.columnconfigure(2, weight=1)
    
      # ************ column 0, 1 **********************

      l = theme.Label(self, text="Entities:")
      l.grid(row=0, column=0, sticky="W", padx=(15,0), pady=(15,0))
      lb = self.lb_entities = theme.Listbox(self, selectmode="browse", width=35, height=20)
      lb.grid(row=1, column=0, sticky="WESN", padx=(15,0), pady=(10,0))
      sr = ListboxScrollbar(lb)
      sr.grid(row=1, column=1, sticky="SN", padx=(2,0), pady=(10,0))           
      self.b_entities = theme.Button(self, text="Add / Remove", width=15, command=self.on_select_entities)
      self.b_entities.grid(row=2, column=0, pady=(10,0))
      
      # *************** column 2, 3 **********************

      l = theme.Label(self, text="Element data:")
      l.grid(row=0, column=2, sticky="W", padx=(15,0), pady=(15,0))
      lb = self.lb_data = theme.Listbox(self, selectmode="browse", width=40, height=20)
      lb.grid(row=1, column=2, sticky="WESN", padx=(15,0), pady=(10,0))
      sr = ListboxScrollbar(lb)
      sr.grid(row=1, column=3, sticky="SN", padx=(2,15), pady=(10,0))                 
      self.b_data = theme.Button(self, text="Add / Remove", width=15, command=self.on_select_data)
      self.b_data.grid(row=2, column=2, pady=(10,0))

      # *********** row 3 ************************

      f = theme.Frame(self)
      f.grid(row=3, column=0, columnspan=4, sticky="EW")
      f.columnconfigure(0, weight=1)      
      b = self.wfa_button = WaitForActionsButton(f, dialog=self)
      b.grid(row=0, column=0, sticky="W", padx=(30,0))
      b = theme.Button(f, text="Ok", width=8, command=self.on_ok)
      b.grid(row=0, column=1, pady=(30,20))
      b = theme.Button(f, text="Cancel", width=8, command=self.on_cancel)
      b.grid(row=0, column=2, padx=(30,20), pady=(30,30))
      
      # *********** events **************************

      self.bind("<Escape>", self.on_cancel)
      self.bind("<Return>", self.on_ok)
      
      # *********** initialize ***********************

      self.set_geometry()
            
      self.lb_entities.focus()
      self.btn_OK = False


   def set_inputs(self, physicals_names_dict, data_refs):
      self.physicals_names_dict = physicals_names_dict
      self.data_refs = data_refs
      self.sel_physicals_ids = []
      self.sel_data_refs = []
      

   def on_select_entities(self, event=None):
      d = EntitySelectionDialog(self, "Entities selection")
      a = {id:v for id, v in self.physicals_names_dict.items() if v[1] == 2}
      d.set_inputs(a, self.sel_physicals_ids)
      if d.execute_modal():
         self.sel_physicals_ids = d.out_selected_ids
         self.lb_entities.delete(0, "end")
         self.lb_entities.insert(0, *d.out_selected_strings)
   

   def on_select_data(self, event=None):
      d = DataSelectionDialog(self, "Element data selection", selectmode="multiple")
      d.set_inputs(self.data_refs, self.sel_data_refs)
      if d.execute_modal():
         self.sel_data_refs = d.out_selected_refs
         self.lb_data.delete(0, "end")
         self.lb_data.insert(0, *d.out_selected_strings)
      

   def on_cancel(self, events=None):
      self.destroy()
   

   def on_ok(self, event=None):
      if len(self.sel_physicals_ids) == 0:
         msg_error(self, "No entity is selected.")
         self.b_entities.focus()
         return
      self.out_sel_physicals_ids = self.sel_physicals_ids
      if len(self.sel_data_refs) == 0:
         msg_error(self, "No data is selected.")
         self.b_data.focus()
         return
      self.out_sel_data_refs = self.sel_data_refs
      self.out_wfa_ids = self.wfa_button.selected_actions_ids
      self.btn_OK = True
      self.destroy()


   # ************* executing ******************

   def execute_modal(self):
      self.wait_window()
      return self.btn_OK




