
from .entity_select import EntitySelectionDialog
from workflow_GUI import theme
from workflow_GUI.messagebox import msg_error

class ShowEntitiesDialog(EntitySelectionDialog):

   def __init__(self, master, title):
      super().__init__(master, title, close_button=True)
      F = self.frame
           
      b = theme.Button(F, text="Get Parameters", command=self.on_calculate)
      b.grid(row=0, column=0, pady=(30,0))

      R = theme.Frame(F)
      R.grid(row=1, column=0, padx=15, pady=10)
      lb = self.R_labs = [4*[None] for _ in range(4)]
      for r in range(4):
         for c in range(4):
            lb[r][c] = theme.Label(R)
            lb[r][c].grid(row=r, column=c, sticky="E", padx=10, pady=10)
      lb[0][1]["text"] = "Minimum"
      lb[0][2]["text"] = "Maximum"
      lb[0][3]["text"] = "Range"
      lb[1][0]["text"] = "x"
      lb[2][0]["text"] = "y"
      lb[3][0]["text"] = "z"

      self.len_area_lab = theme.Label(F)
      self.len_area_lab.grid(row=2, column=0, sticky="W", padx=15, pady=10)
      self.volume_lab = theme.Label(F)
      self.volume_lab.grid(row=3, column=0, sticky="W", padx=15, pady=10)

      self.b_protocol = theme.Button(F, text="Write to protocol", command=self.on_write_to_protocol)
      self.b_protocol.grid(row=4, column=0, pady=15)     
      self.b_protocol["state"] = "disable"

      self.sel_var.trace_add("write", self.reset_info)

      # *************** initialize ***************
      
      self.set_geometry()



   def on_calculate(self, event=None):
      ids = self.get_sel_ids()
      if len(ids) == 0:
         msg_error(self, "No entities were selected.")
         return
      m, action_name = self.master, self.title()
      self.last_thread = m.perform_isolated_action(action_name, m.diskmod.report_physicals_group, ids)     
      if self.last_thread is None:
         msg_error(self, "Calculation error")
         return
      d = self.last_thread.result[0]
      lb, f = self.R_labs, m.float2str
      for r in range(3):
         for c in range(3):
            lb[r+1][c+1]["text"] = f(d[r][c], "coordinate")
      if d[3] is not None: s = "Length: " + f(d[3], "area")
      elif d[4] is not None: s = "Area: " + f(d[4], "area")
      else: s = ""
      self.len_area_lab["text"] = s
      self.volume_lab["text"] = "Volume: " + f(d[5], "volume")
      self.b_protocol["state"] = "normal"


   def on_write_to_protocol(self):
      t = self.last_thread
      t.action_id = self.master.gen_action_id()
      p_in =[("Entities IDs", self.get_sel_ids())]
      r = t.result[1]
      p = [(k, self.master.float2str(v, "coordinate")) for k, v in r[:9]]
      if r[9][1] is not None:
         p.append((r[9][0], self.master.float2str(r[9][1], "area")))
      if r[10][1] is not None:
         p.append((r[10][0], self.master.float2str(r[10][1], "area")))
      if r[11][1] is not None:
         p.append((r[11][0], self.master.float2str(r[11][1], "volume")))
      self.master.protocol_perform_action(t, inputs=p_in, results=p)


   def reset_info(self, n, i, m):
      lb = self.R_labs
      for r in range(3):
         for c in range(3): lb[r+1][c+1]["text"] = ""
      self.len_area_lab["text"] = ""
      self.volume_lab["text"] = ""
      self.b_protocol["state"] = "disable"



