import tkinter as tk
from workflow_GUI import theme

class ShowModelRangeDialog(theme.Toplevel):

   def __init__(self, master, title):
      super().__init__(master)
      self.grab_set()
      self.title(title)
      self.resizable(False, False)

      frame = theme.Frame(self)
      frame.grid(column=0, row=0, columnspan=3, padx=20, pady=20)
      lb = self.lab = [4*[None] for _ in range(5)]
      for r in range(5):
         for c in range(4):
            lb[r][c] = theme.Label(frame)
            lb[r][c].grid(row=r, column=c, sticky="E", padx=10, pady=10)
      lb[0][1]["text"] = "Minimum"
      lb[0][2]["text"] = "Maximum"
      lb[0][3]["text"] = "Range"
      lb[1][0]["text"] = "x"
      lb[2][0]["text"] = "y"
      lb[3][0]["text"] = "z"
      lb[4][0]["text"] = "DEM z"

      self.to_protocol_var = tk.BooleanVar(self, value=False)
      ch = self.check_to_protocol = theme.Checkbutton(self, text="Write to protocol", variable=self.to_protocol_var)
      ch.grid(column=0, row=1, sticky="w", padx=(20,0), pady=20)

      b = theme.Button(self, text="OK", width=8, command=self.on_ok)
      b.grid(column=1, row=2, padx=20, pady=20)
      b = theme.Button(self, text="Cancel", width=8, command=self.on_cancel)
      b.grid(column=2, row=2, padx=20, pady=20)


      # *********** events **************************
      self.bind("<Escape>", self.on_cancel)
      self.bind("<Return>", self.on_ok)

      # ************ initialize ********************

      self.set_geometry(resizable=False)

      self.check_to_protocol.focus()
      self.btn_OK = False

      
   def set_inputs(self, mesh_range, DEM_range):
      lb = self.lab
      for r in range(3):
         for c in range(3):
            lb[r+1][c+1]["text"] = self.master.float2str(mesh_range[r][c], "coordinate")
      if DEM_range is not None:
         for c in range(3):
            lb[4][c+1]["text"] = self.master.float2str(DEM_range[c], "coordinate")


   def on_ok(self, event=None):
      self.out_write_to_protocol = self.to_protocol_var.get()
      self.btn_OK = True
      self.destroy()

   def on_cancel(self, event=None):
      self.destroy()


   # ************* executing ******************

   def execute_modal(self):
      self.wait_window()
      return self.btn_OK
