import tkinter as tk
import tkinter.filedialog as filedialog

from workflow_GUI import theme
from workflow_GUI.messagebox import msg_error


# ***********************************************

class FilenameFrame(theme.Frame):
   
   def __init__(self, master, title="File name:", title_above=False, allow_empty=False):
      super().__init__(master)
      self.allow_empty = allow_empty
      
      self.columnconfigure(1, weight=1)
      self.label = theme.Label(self, text=title)
      if title_above: self.label.grid(row=0, column=1, sticky="W")
      else: self.label.grid(row=1, column=0, sticky="W")
      self.file_var = tk.StringVar(self)
      self.file_e = theme.Entry(self, textvariable=self.file_var)
      self.file_e.grid(row=1, column=1, sticky="WE", padx=(0 if title_above else 20 ,8))
      self.file_dlg_b = theme.Button(self, text="...", width=3, command=self.on_choose_file_name)
      self.file_dlg_b.grid(row=1, column=2, sticky="E")

   def set_open_dialog(self, **opt):
      self.dlg_type = "open"
      self.dlg_opt = opt
               
   def set_save_dialog(self, **opt):
      self.dlg_type = "save"
      self.dlg_opt = opt

   def get_value(self):
      s = self.file_var.get().strip()
      if s == "":
         if self.allow_empty: return
         msg_error(self.master, "File name was not set.")
         self.file_e.focus()
         return
      else:
         return s

   def on_choose_file_name(self):
      d = self.__dict__
      if self.dlg_type == "open":
         file_name = filedialog.askopenfilename(**self.dlg_opt)
      if self.dlg_type == "save":
         file_name = filedialog.asksaveasfilename(**self.dlg_opt)
      self.file_var.set(file_name)
      
      
# ************************************************

class ListboxScrollbar(theme.Scrollbar):
   
   def __init__(self, listbox, width=20):
      super().__init__(listbox.master, width=width, command=listbox.yview)
      listbox["yscrollcommand"] = self.set


# **************************************************

class RangeFrame(theme.Frame):
   
   def __init__(self, master, is_DEM):
      super().__init__(master, relief="raised", bd=2)

      l = theme.Label(self, text="Range settings")
      l.grid(row=0, column=0, columnspan=5)
       
      w = lambda text: (
         theme.Label(self, text=text, anchor="w"),
         v:=tk.StringVar(self),
         theme.Entry(self, textvariable=v, width=10) )

      l, self.min_x, self.e_min_x = w("X min.:")
      l.grid(row=1, column=0, sticky="w", padx=(10,0), pady=(5,0))
      self.e_min_x.grid(row=1, column=1, sticky="w", padx=(10,0), pady=(5,0))
      l, self.max_x, self.e_max_x = w("X max.:")
      l.grid(row=1, column=2, sticky="w", padx=(30,0), pady=(5,0))
      self.e_max_x.grid(row=1, column=3, sticky="w", padx=(10,0), pady=(5,0))

      l, self.min_y, self.e_min_y = w("Y min.:")
      l.grid(row=2, column=0, sticky="w", padx=(10,0), pady=(5,0))
      self.e_min_y.grid(row=2, column=1, sticky="w", padx=(10,0), pady=(5,0))
      l, self.max_y, self.e_max_y = w("Y max.:")
      l.grid(row=2, column=2, sticky="w", padx=(30,0), pady=(5,0))
      self.e_max_y.grid(row=2, column=3, sticky="w", padx=(10,0), pady=(5,0))

      self.xy_by_ref = tk.BooleanVar(self)
      ch = theme.Checkbutton(self, text="Use x, y as offset of reference point", variable=self.xy_by_ref)
      ch.grid(row=3, column=0, columnspan=4, pady=(5,0))

      l, self.min_z, self.e_min_z = w("Z min.:")
      l.grid(row=4, column=0, sticky="w", padx=(10,0), pady=(5,0))
      self.e_min_z.grid(row=4, column=1, sticky="w", padx=(10,0), pady=(5,0))
      l, self.max_z, self.e_max_z = w("Z max.:")
      l.grid(row=4, column=2, sticky="w", padx=(30,0), pady=(5,0))
      self.e_max_z.grid(row=4, column=3, sticky="w", padx=(10,0), pady=(5,0))

      l, self.min_depth, self.e_min_depth = w("Depth min.:")
      l.grid(row=5, column=0, sticky="w", padx=(10,0), pady=(5,10))
      self.e_min_depth.grid(row=5, column=1, sticky="w", padx=(10,0), pady=(5,10))
      l, self.max_depth, self.e_max_depth = w("Depth max.:")
      l.grid(row=5, column=2, sticky="w", padx=(30,0), pady=(5,10))
      self.e_max_depth.grid(row=5, column=3, sticky="w", padx=(10,0), pady=(5,10))

      self.include_style = tk.StringVar(self, value="center")
      rb = theme.Radiobutton(self, text="Must include element center", variable=self.include_style, value="center")
      rb.grid(row=1, column=4, sticky="W", padx=(15,0))
      rb = theme.Radiobutton(self, text="Can include any element part", variable=self.include_style, value="any")
      rb.grid(row=2, column=4, sticky="W", padx=(15,10))

      self.depth_relation = tk.StringVar(self, value="top_boundary")
      rb = theme.Radiobutton(self, text="Depth relative to top boundary", variable=self.depth_relation, value="top_boundary")
      rb.grid(row=4, column=4, sticky="W", padx=(15,0))      
      rb = theme.Radiobutton(self, text="Depth relative to DEM", variable=self.depth_relation, value="DEM")
      rb.grid(row=5, column=4, sticky="W", padx=(15,0), pady=(0,10))
      if not is_DEM: rb["state"] = "disable"


   def _get_range(self, var1, e1, var2, e2, var_name):
      a1 = var1.get().strip()
      if a1 != "": 
         try: a1 = float(a1)
         except:
            msg_error(self.master, "Minimum value of "+var_name+" range is not valid.")
            e1.focus()
            return
      else: a1 = None
      a2 = var2.get().strip()
      if a2 != "": 
         try: a2 = float(a2)
         except:
            msg_error(self.master, "Maximum value of "+var_name+" range is not valid.")
            e2.focus()
            return
      else: a2 = None
      if a1 is not None and a2 is not None:
         if a1 >= a2:
            msg_error(self.master, "Maximum value of "+var_name+" range must be greater than minimum value.")
            e2.focus()
            return
      return a1, a2
         

   def get_values(self):
      if (x := self._get_range(self.min_x, self.e_min_x, self.max_x, self.e_max_x, "X")) is None: return
      if (y := self._get_range(self.min_y, self.e_min_y, self.max_y, self.e_max_y, "Y")) is None: return
      if (z := self._get_range(self.min_z, self.e_min_z, self.max_z, self.e_max_z, "Z")) is None: return
      if (d := self._get_range(self.min_depth, self.e_min_depth, self.max_depth, self.e_max_depth, "depth")) is None: return
      c = self.include_style.get() == "center"
      return {"x_min":x[0], "x_max":x[1],
              "y_min":y[0], "y_max":y[1],
              "xy_by_ref":self.xy_by_ref.get(),
              "z_min":z[0], "z_max":z[1],
              "depth_min":d[0], "depth_max":d[1],
              "depth_by_DEM": self.depth_relation.get() == "DEM",
              "center_in_range":c}
            


