

def toolkit_order_generator():
   i = -1
   while True:
      i += 1
      yield i

gen_toolkit_order = toolkit_order_generator()

class MeshToolKit:

   class ToolKitException(Exception): pass

   def __init__(self, **kwargs):
      info = kwargs.get("info")
      if info is not None: kwargs.pop("info")
      mesh = kwargs.get("mesh")
      if mesh is not None: kwargs.pop("mesh")
      self.name = kwargs.get("name")
      if self.name is not None: kwargs.pop("name")
      else: self.name = self.__class__.__name__
      toolkits = kwargs
      self.order = next(gen_toolkit_order)
      self.above_toolkits = []
      if len(toolkits) > 0:
         for name, t in toolkits.items(): 
            self.__dict__[name] = t
            t.above_toolkits.append(self)
         self.info = t.info
         self.mesh = t.mesh

      else:
         self.info = mesh.info if info is None else info
         self.mesh = mesh
      if self.mesh is not None: self.mesh.above_toolkits.append(self)


   def get_above_ordered_toolkits(self):
      aboves = set()
      z = [(t for t in self.above_toolkits + [None])]
      while len(z) > 0:
         a = next(z[-1])
         if a is not None:
            aboves.add(a)
            if len(a.above_toolkits) > 0:
               z.append((t for t in a.above_toolkits + [None]))
         else: z.pop()
      return sorted(aboves, key=lambda t: t.order) 
         

   def update_above_toolkits(self):
      self.info.start("Updating toolkits above: " + self.name)
      aboves = self.get_above_ordered_toolkits()
      for t in aboves: t.update()
      self.info.OK()


   def update(self, **opt):
      if len(opt) > 0:
         for k, v in opt.items(): self.__dict__["OPTION_"+k] = v
      if "update_method" in dir(self):
         self.info.start("Updating toolkit: " + self.name)
         self.update_method()
         self.info.OK()      

