
import tkinter as tk
from tkinter import filedialog
import datetime, inspect, traceback
import os, sys

import yaml

from . import theme
from .servis import ActionFrame, Info, RUNNING, COMPLETED, TERMINATED, ACTION_ID_ISOLATED
from .messagebox import msg_info, msg_warning, msg_error, msg_yesno, msg_yesnocancel, msg_okcancel, msg_cancel
from .dlg_log_file import LogFileDialog


class BaseApplication(tk.Tk):
   
   def __init__(self, option_file_name):
      super().__init__()

      # *********** initial options *******************
      with open(self.extent_file_name(option_file_name)) as f:
         self.options = opt = yaml.safe_load(f)           
      if (s:=opt["basics"]["theme file"]) is not None: theme.load_theme(self.extent_file_name(s))
      self.actions_log_file = self.extent_file_name(opt["basics"]["actions log file"])
      self.OPTION_menu_style = tk.StringVar(self, "menubar")
      self.OPTION_log_position = tk.StringVar(self, "bottom")
      self.OPTION_action_threads_max_count = opt["start"]["maximum threads number"]

      # ***** initial atribute values **************
      self.set_protocol_key_names()

      # *** main window base initialization *****
      self.title("Model Workflow Tools")
      self.geometry("1000x700+100+100")
      self.columnconfigure(0, weight=1)
      self.rowconfigure(0, weight=0)
      self.rowconfigure(1, weight=1)
      self.rowconfigure(2, weight=0)
          
      # ********* actions *********************
      self.info = Info(self)
      self.action_re_method = {}
      self.on_method_name_action = {}
      self.action_id_generator = self.ActionIdGemerator(0)
      self.actions_threads = []
      self.bind("<<action_thread_info>>", self.on_action_thread_info)
      self.bind("<<action_thread_start>>", self.on_action_thread_start)
      self.bind("<<action_thread_completed>>", self.on_action_thread_completed)
      self.bind("<<action_thread_terminated>>", self.on_action_thread_terminated)

      # ********** creating *******************      
      self.create_menu()
      self.create_toolbar()
      self.create_workarea()
      self.create_statusbar()


   def do_refresh_status(self): pass
   def do_after_start(self): pass
   def do_after_protocol_delete(self): pass
   def do_after_protocol_perform(self): pass
   
   def initialize_start_state(self, start_protocol_file_name):
      self.start_protocol_file_name = start_protocol_file_name
      self.on_change_menu_style()      
      self.bind("<<start_state>>", self.on_start_state)
      self.update()
      self.event_generate("<<start_state>>", when="tail")


   def on_start_state(self, event):
      self.unbind("<<start_state>>")
      self.after(100, self.on_refresh_status)
      self.do_after_start()
      file_name = self.start_protocol_file_name
      if file_name is not None: self.perform_protocol(file_name)
      else: self.on_protocol_new()           
      
   def on_refresh_status(self):
      self.do_refresh_status()
      self.after(250, self.on_refresh_status)


   def set_protocol_key_names(self,
         start_action = "Starting action",
         complete_action = "Completing action",
         perform_action = "Performed action",
         action_id = "ID",
         wait_for_actions = "Wait for actions",
         inputs = "Input params",
         options = "Options",
         results = "Results" ):
      self.pk_start_action = start_action
      self.pk_complete_action = complete_action
      self.pk_perform_action = perform_action
      self.pk_action_id = action_id
      self.pk_wait_for_actions = wait_for_actions
      self.pk_inputs = inputs
      self.pk_options = options
      self.pk_results = results

      
   # ******** creating main window *****************

   def create_menu(self):     
      self.mainmenu = m = theme.Menu(self, tearoff=False)
      self.config(menu=m)     
      m.add_cascade(label="Protocol", menu=(mp:=theme.Menu(m, tearoff=0)))
      mp.add_command(label="New", command=self.on_protocol_new)
      mp.add_command(label="Open and Execute", command=self.on_protocol_open_and_perform)
      mp.add_command(label="Save", command=self.on_protocol_save)
      mp.add_command(label="Save As", command=self.on_protocol_save_as)
      mp.add_separator()
      mp.add_command(label="Exit", command=self.destroy, accelerator="Ctrl+A")
      m.add_cascade(label="Options", menu=(mo:=theme.Menu(m, tearoff=0)))
      mo.add_cascade(label="Menu Style", menu=(mo_ms:=theme.Menu(mo, tearoff=0)))
      mo_ms.add_radiobutton(label="Menu Bar", variable=self.OPTION_menu_style, value="menubar", command=self.on_change_menu_style)
      mo_ms.add_radiobutton(label="Popup Menu", variable=self.OPTION_menu_style, value="popup", command=self.on_change_menu_style)
      mo.add_cascade(label="Log Position", menu=(mo_lp:=theme.Menu(mo, tearoff=0)))
      mo_lp.add_radiobutton(label="Bottom", variable=self.OPTION_log_position, value="bottom", command=self.on_update_workarea)
      mo_lp.add_radiobutton(label="Right", variable=self.OPTION_log_position, value="right", command=self.on_update_workarea)
      mo.add_command(label="Show Log File", command=self.on_show_log_file)


   def create_toolbar(self):
      t = self.toolbar = theme.ToolBar_Frame(self, height=20)
      t.grid(row=0, column=0, sticky="SNEW")
      t.button_menu = theme.ToolBar_Button(t, text="Menu", command=self.on_popup_menu)

          
   def create_workarea(self):
      a = self.workarea = theme.PanedWindow(self, orient="vertical")
      a.grid(row=1, column=0, sticky="SNEW")      
      self.protocol = theme.Protocol_ScrolledText(a, wrap=tk.NONE, height=15, width=40)
      a.add(self.protocol)
      a.actions = theme.PanedWindow(a, orient='horizontal')
      a.add(a.actions)


   def create_statusbar(self):
      s = self.statusbar = theme.StatusBar_Frame(self, height=40, relief="groove", bd=3)
      s.grid(row=2, column=0, sticky="SNEW")


   # ********** general support ***********
   
   def extent_file_name(self, file_name):
      if os.path.split(file_name)[0] == "":
         app_dir = os.path.realpath(sys.path[0])         
         file_name = os.path.join(app_dir, file_name)
      return file_name
     
   def float2str(self, x, format=None):
      if format is not None:
         r = self.options["number formats"][format]
         s = str(x) if r is None or r == "" else ("{:"+r+"}").format(x)
      else: s = str(x)
      while s[-1] == "0" and s != "0": s = s[:-1]
      if s[-1] == ".": s = s[:-1]
      return s


   # ******* action support ****************

   def add_actions_menu_items(self, menu, items):
      for label, command, shortcut in items:
         if label == "-": menu.add_separator()
         elif command is None: menu.add_command(label=label)
         else:
            e = eval("self.on_"+command)
            menu.add_command(label=label, command=e)         
            self.action_re_method[label] = eval("self.re_"+command) if hasattr(self, "re_"+command) else None
            self.on_method_name_action["on_"+command] = label


   def get_action_name_by_myname(self):
      return self.on_method_name_action[inspect.stack()[1].function]


   def gen_action_id(self):
      return next(self.action_id_generator)


   def ActionIdGemerator(self, start_id):
      id = start_id
      while True:
         yield id
         id += 1


   # **********************************************
   # ************ option menu **********************

   # ******* menu events ************************

   def on_change_menu_style(self):
      if self.OPTION_menu_style.get() == "menubar":
         self.config(menu=self.mainmenu)
         self.toolbar.button_menu.grid_forget()
         self.bind("<F10>", self.on_focus_menubar)
      else:
         self.config(menu="")
         self.toolbar.button_menu.grid(row=0, column=0)
         self.bind("<F10>", self.on_popup_menu)

   def on_popup_menu(self, event=None):
      self.mainmenu.tk_popup(self.toolbar.button_menu.winfo_rootx(), self.toolbar.button_menu.winfo_rooty() + self.toolbar.button_menu.winfo_height(), "")
    
   def on_focus_menubar(self, event=None):
      self.mainmenu.focus()


   # ******** change log styles *************************

   def on_update_workarea(self):
      if self.OPTION_log_position.get() == "bottom":
         self.workarea.config(orient="vertical")
         self.workarea.actions.config(orient="horizontal")
      else:
         self.workarea.config(orient="horizontal")
         self.workarea.actions.config(orient="vertical")
   

   # ************ show log file ******************

   def on_show_log_file(self):
      d = self.log_file_dialog = LogFileDialog(self)
      with open(self.actions_log_file) as f:
         d.text.insert("0.0", f.read())
      d.text.see("end")     
      if d.execute_modal():
         pass
      delattr(self, "log_file_dialog")


   # ***** protocol file name and title ***********

   def set_app_name(self, name):
      self.app_name = name
      self.title(name)
      
   def set_protocol_file_name(self, name):
      self.protocol_file_name = name
      s = name if name != "" else "No name"
      self.title(self.app_name + "  - protocol: " + s)
    
         
   # ***********************************************
   # ********* protocol menu ***********************

   def on_protocol_new(self):
      if self.is_action_runing(): return
      save_change = False
      if (p:=self.protocol).edit_modified():
         save_change = msg_yesnocancel(self, self.protocol_file_name+"\nDo you want save changes?", title="Saving changes")
         if save_change is None: return
         elif save_change: self.on_protocol_save()         
      if not p.edit_modified() or not save_change:
         p.delete("1.0", tk.END)
         p.edit_modified(False)         
         self.do_after_protocol_delete()
         self.set_protocol_file_name("")
         self.actions_threads.clear()
         self.action_id_generator = self.ActionIdGemerator(0)


   def on_protocol_open_and_perform(self):
      if self.is_action_runing(): return
      save_change = False
      if (p:=self.protocol).edit_modified():
         save_change = msg_yesnocancel(self, self.protocol_file_name+"\nDo you want to save changes in actual protocol?", title="Saving changes")
         if save_change is None: return
         elif save_change: self.on_protocol_save()         
      if not self.protocol.edit_modified() or not save_change:
         file_name = filedialog.askopenfilename(
            title="Open protocol",
            filetypes=[("protocol yaml file", ".yaml")],
            defaultextension=".yaml",
            initialdir=".")
         if file_name != "":
            self.perform_protocol(file_name)

            
   def perform_protocol(self, file_name):      
      p = self.protocol
      p.delete("1.0", tk.END)
      p.edit_modified(False)
      self.do_after_protocol_delete()
      with open(file_name) as f:
         protocol = yaml.safe_load(f)
      self.set_protocol_file_name(file_name)
      self.actions_threads.clear()
      self.action_id_generator = self.ActionIdGemerator(0)    
      try: actions = [(a, a_name) for a in protocol if (a_name := a.get(self.pk_start_action, a.get(self.pk_perform_action, None))) is not None]
      except:
         msg_error(self, "Error of reading protocol")
         return
      d = self.opening_dialog = msg_cancel(self, "Opening protocol: '" + self.protocol_file_name + "' ...")
      self.after(0, self.on_perform_protocol_next_action, actions)      
      d.wait_window()
      if d.result is None:
         for t in self.actions_threads:
            if t.state == RUNNING: t.user_termination = True
      d = None

   def on_perform_protocol_next_action(self, actions):
      if self.opening_dialog.result is None: return
      if len(self.actions_threads) > 0:
         if self.actions_threads[-1].state == TERMINATED:
            self.opening_dialog.destroy()
            return
      if self.get_running_action_threads_count() < self.OPTION_action_threads_max_count:
         complete = {t.action_id for t in self.actions_threads if t.state==COMPLETED}
         for p in actions:
            a, a_name = p
            if len(set(a.get(self.pk_wait_for_actions, [])) - complete) == 0:
               actions.remove(p)
               id_name_wait = (id:=a[self.pk_action_id], a_name, a.get(self.pk_wait_for_actions))
               if self.action_re_method[a_name] is not None:                
                  try: self.action_re_method[a_name](id_name_wait, a)
                  except:
                     self.opening_dialog.destroy()
                     print(traceback.format_exc())
                     msg_error(self, "Action: '"+a_name+"'  ID "+(str(id) if "id" in locals() else "unknown")+"\nError of evaluation action params from protocol.")
                     return                  
               else:
                  self.protocol_re_not_supported(id_name_wait)
               while id > self.gen_action_id(): pass
               break
      if len(actions) > 0: self.after(200, self.on_perform_protocol_next_action, actions)      
      else: self.after(100, self.on_finish_perform_protocol)
         
   def on_finish_perform_protocol(self):
      if self.get_running_action_threads_count() > 0:
         self.after(100, self.on_finish_perform_protocol)
      else:
         self.opening_dialog.destroy()
         self.after(100, self.do_after_protocol_perform)

           
   def on_protocol_save(self):
      if self.is_action_runing(): return
      if self.protocol_file_name == "": 
         self.on_protocol_save_as()
      else:
         with open(self.protocol_file_name, "w") as f:
            f.write(self.protocol.get("1.0", tk.END))
         self.protocol.edit_modified(False)
        

   def on_protocol_save_as(self):
      if self.is_action_runing(): return
      f = filedialog.asksaveasfile(
         title="Save protocol as",
         filetypes=[("protocol yaml file", ".yaml")],
         defaultextension=".yaml",
         initialdir=".",
         initialfile="",
         confirmoverwrite=True)
      if f is not None:
         try: f.write(self.protocol.get("1.0", tk.END))
         finally: f.close()
         self.set_protocol_file_name(f.name)
         self.protocol.edit_modified(False)


   def destroy(self):
      if self.is_action_runing(): return
      save_change = False
      if (p:=self.protocol).edit_modified():
         save_change = msg_yesnocancel(self, self.protocol_file_name+"\nDo you want to save changes?", title="Saving changes")
         if save_change is None: return
         elif save_change: self.on_protocol_save()
      if not self.protocol.edit_modified() or not save_change:
         super().destroy()
      
     
   def is_action_runing(self):
      if b:=any(not f.log_saved for f in self.get_actions_frames()):
         msg_info(self, "Operation cannot be processed, \nbecause some actions are still runnig.", title="Actions runnig")
      return b




   # ********** protocol write *******************

   def protocol_write_line(self, line):
      self.protocol.insert(tk.END, line + "\n")
      self.protocol.see(tk.END)


   def protocol_write_data(self, data):
      n = max(len(d[0]) for d in data)
      for key, value in data:
         if (t:=type(value)) is not str:
            if t is bool: value = "yes" if value else "no"
            elif t is float: value = str(value)
            elif t in (list, tuple, set): value = "[" + ", ".join(["'"+v+"'" if type(v) is str else str(v) for v in value]) + "]"
            else: value = str(value)
         if key.strip() == "-":
            self.protocol_write_line("  " + key + value)
         else:
            i = len(key) - 1
            while key[i] == " ": i -= 1
            k = len(key) - i - 1
            if k > 0: self.protocol_write_line(
               "    " + key + ":" + k*" " + value )
            else: self.protocol_write_line(
               "    " + key + ":" + (n-len(key)+1)*" " + value )
                                   

   def protocol_start_action(self, action_thread, inputs=None):
      w = self.protocol_write_line
      w("- " + self.pk_start_action + ": " + action_thread.action_name)
      w("  Time: " + str(action_thread.start_time).split(".")[0])
      w("  " + self.pk_action_id + ": " + str(action_thread.action_id))
      if (a:=action_thread.wait_for_actions_ids) is not None:
         w("  " + self.pk_wait_for_actions + ": " + str(a))
      if inputs is not None:
         w("  " + self.pk_inputs + ":")
         self.protocol_write_data(inputs)
      w("")


   def protocol_complete_action(self, action_thread, results=None):
      w = self.protocol_write_line
      w("- " + self.pk_complete_action + ": " + action_thread.action_name)
      w("  Duration time: " + (t:=str(action_thread.duration_time).split("."))[0] + "." + t[1][:-2])
      w("  " + self.pk_action_id + ": " + str(action_thread.action_id))
      if results is not None:         
         if callable(results):            
            results = results(action_thread.result)
         if results is not None:
            w("  " + self.pk_results + ":")
            self.protocol_write_data(results)
      w("")


   def protocol_terminate_action(self, action_thread, write_action_id=True, results=None, dependences=None):
      w, r = self.protocol_write_line, action_thread
      if r.is_terminated_by_user: w("- User terminated action: " + r.action_name)
      else: w("- ERROR of action: " + r.action_name)
      w("  Duration time: " + str(r.duration_time).split(".")[0])
      w("  " + self.pk_action_id + ": " + str(r.action_id))
      w("")


   def protocol_perform_action(self, action_thread, action_name=None, wait_for_actions_ids=None, options=None, inputs=None, results=None):
      w, r = self.protocol_write_line, action_thread
      if r is None:
         class R: pass
         r = R()
         r.action_name = action_name
         r.wait_for_actions_ids = wait_for_actions_ids
         r.action_id = self.gen_action_id()
         r.start_time = datetime.datetime.now()
      w("- " + self.pk_perform_action + ": " + r.action_name)
      w("  Time: " + str(r.start_time).split(".")[0])
      w("  " + self.pk_action_id + ": " + str(r.action_id))
      if (a:=r.wait_for_actions_ids) is not None:
         w("  " + self.pk_wait_for_actions + ": " + str(a))
      if options is not None:
         w("  " + self.pk_options + ":")
         self.protocol_write_data(options)
      if inputs is not None:
         w("  " + self.pk_inputs + ":")
         self.protocol_write_data(inputs)
      if results is not None:         
         w("  " + self.pk_results + ":")
         if callable(results): results = results(r.result)         
         self.protocol_write_data(results)
      w("")


   def protocol_re_not_supported(self, id_name_wait):
      w, id, name, idw = self.protocol_write_line, *id_name_wait
      w("- WARNING: action reconstruction is not supported")
      w("  Action name: " + name)
      w("  ID: " + str(id))
      if idw is not None: w("  Wait for action: " + str(idw))
      w("")

   
   # ********** action method *************************

   def get_actions_frames(self):
      return [self.nametowidget(p) for p in self.workarea.actions.panes()]

   def get_running_action_threads_count(self):
      return sum(1 for t in self.actions_threads if t.state == RUNNING)

   def get_action_frame_by_id(self, action_id):
      return next(f for f in self.get_actions_frames() if f.thread.action_id == action_id)

   def create_action_frame(self, action_id, action_name, wait_for_actions_ids, func, *args, **kwargs):
      f = ActionFrame(self, action_id, action_name, wait_for_actions_ids, func, *args, **kwargs)
      self.actions_threads.append(f.thread)
      self.workarea.actions.add(f)
      while (f_0:=self.get_actions_frames()[0]).log_saved:
         self.workarea.actions.remove(f_0)
      return f


   def perform_isolated_action(self, action_name, func, *args, **kwargs):
      f = ActionFrame(self, ACTION_ID_ISOLATED, action_name, None, func, *args, **kwargs)
      self.workarea.actions.add(f)
      while (f_0:=self.get_actions_frames()[0]).log_saved:
         self.workarea.actions.remove(f_0)
      gb = f.last_grab = self.grab_current()
      if gb is not None:
         self.focus()
      f.grab_set_global()
      d = theme.Toplevel(self, relief="groove", bd=6)
      theme.Label(d, text="Processing isolated action, please wait ....").grid(row=0, column=0, padx=60, pady=30)   
        # self.attributes('-toolwindow', True)
      d.wm_overrideredirect(True)   
      d.set_geometry(resizable=False)
      f.thread.start()
      ok = f.thread.wait()     
      d.destroy()
      if gb is None: f.grab_release()
      else:
         gb.focus()
         gb.grab_set()         
      return f.thread if ok else None


   def on_action_thread_info(self, event):
      f_t, s = self.info.queue.get()
      if "\b" in s:
         for c in s:
            if c == "\b": f_t.delete("end - 2 chars", "end")
            else: f_t.insert("end", c)
      else: 
         f_t.insert("end", s)
      f_t.see(tk.END)


   def on_action_thread_start(self, event):
      f = self.get_action_frame_by_id(event.state)
      if f.do_before_start is not None: f.do_before_start(f.thread, *f.do_before_start_args, **f.do_before_start_kwargs)


   def on_action_thread_completed(self, event):      
      f = self.get_action_frame_by_id(event.state)
      f.stop_button.destroy()
      if "log_file_dialog" not in self.__dict__: self.save_actions_logs()
      if f.do_after_completed is not None: f.do_after_completed(f.thread, *f.do_after_completed_args, **f.do_after_completed_kwargs)


   def on_action_thread_terminated(self, event):      
      f = self.get_action_frame_by_id(event.state)
      f.stop_button.destroy()
      if "log_file_dialog" not in self.__dict__: self.save_actions_logs()
      if f.do_after_terminated is not None: f.do_after_terminated(f.thread, *f.do_after_terminated_args, **f.do_after_terminated_kwargs)

      
   def save_actions_logs(self):
      with open(self.actions_log_file, "a") as g:
         for f in self.get_actions_frames():
            if f.log_saved: continue
            if f.thread.state in (COMPLETED, TERMINATED):
               g.write(f.text.get("0.0", tk.END))
               f.log_saved = True
            else:
               break
      





