import tkinter as tk

from . import theme

class WaitForActionsDialog(theme.Toplevel):

   def __init__(self, master, tk_master, init_selected_actions_ids, title, info_text):
      super().__init__(master)
      self.tk_master = tk_master
      self.last_grab = self.grab_current()
      self.grab_set()
      self.title("Wait for actions" if title is None else title)
      w, h = 600, 500
      self.geometry("{:n}x{:n}+{:n}+{:n}".format(w, h, master.winfo_x()+master.winfo_width()//2-w//2, master.winfo_y()+master.winfo_height()//2-h//2))
      self.minsize(w, h)

      self.rowconfigure(2, weight=1)
      self.columnconfigure(0, weight=1)
      
      if info_text is not None:
         l = theme.Label(self, text=info_text, justify="left")
         l.grid(row=0, column=0, columnspan=2, sticky="W", padx=10, pady=(20,0))

      l = theme.Label(self, text="Waiting for actions selection:")
      l.grid(row=1, column=0, sticky="W", padx=10, pady=(20,10))
      v = self.actions_var = tk.StringVar(self)
      lb = self.actions_lb = theme.Listbox(self, listvar=v, selectmode="multiple")
      lb.grid(row=2, column=0, sticky="SWNE", padx=(10,2), pady=(0,10))
      sr = theme.Scrollbar(self, width=20, command=lb.yview)
      lb["yscrollcommand"] = sr.set
      sr.grid(row=2, column=1, sticky="SN", pady=(0,10))      

      f = theme.Frame(self)
      f.grid(row=2, column=2, sticky="NSWE")
      f.rowconfigure(2, weight=1)
      b = theme.Button(f, text="OK", width=8, command=self.on_ok)
      b.grid(row=0, column=0, padx=(60,20), pady=10)
      self.bind("<Return>", self.on_ok)
      b = theme.Button(f, text="Calcel", width=8, command=self.on_cancel)
      self.bind("<Escape>", self.on_cancel)
      b.grid(row=1, column=0, padx=(60,20), pady=10)
      b = theme.Button(f, text="Clear selection", width=12, command=self.on_clear_selection)
      b.grid(row=2, column=0, sticky="S", padx=20, pady=40)

      if len(self.tk_master.actions_threads) > 0:
         self.set_actions_list()
      if (a:=init_selected_actions_ids) is not None:
         a = set(a)
         for i, s in enumerate(self.actions_lb.get(0, "end")):
            if int(s.split()[0]) in a: self.actions_lb.selection_set(i) 
      self.focus()
      self.actions_lb.focus()
      self.btn_OK = False
      
      
   def set_actions_list(self):      
      sel_i = self.actions_lb.curselection()
      n = max(len(str(t.action_id)) for t in self.tk_master.actions_threads)
      self.actions_var.set([("{:"+str(n)+"n} {:s}  ({:s})").format(t.action_id, t.action_name, t.state) for t in self.tk_master.actions_threads])
      for i in sel_i: self.actions_lb.selection_set(i)
      self.actions_lb.see("end")
      self.after(500, self.set_actions_list)


   def destroy(self):
      if self.last_grab is None: self.grab_release()
      else: self.last_grab.grab_set()
      super().destroy()


   def on_clear_selection(self, event=None):
      self.actions_lb.selection_clear(0, "end")


   def on_cancel(self, event=None):
      self.destroy()


   def on_ok(self, event=None):
      a = self.actions_lb
      if len(a.curselection()) > 0:
         self.out_selected_actions_ids = [int(a.get(i).split()[0]) for i in a.curselection()]
      else:
         self.out_selected_actions_ids = None
      self.destroy()
      self.btn_OK = True



# ******* button for show dialog *********************

class WaitForActionsButton(theme.Button):

   def __init__(self, master, dialog=None, tk_master=None):
      super().__init__(master, text="Wait for actions", command=self.on_click)
      self.dialog = master if dialog is None else dialog
      self.tk_master = self.dialog.master if tk_master is None else tk_master
      self.selected_actions_ids = None
   

   def on_click(self, event=None):
      d = WaitForActionsDialog(self.dialog, self.tk_master, self.selected_actions_ids, None, None)
      d.wait_window()
      if d.btn_OK:
         self.selected_actions_ids = d.out_selected_actions_ids
      

# ************ function for show dialog **************

def get_wait_for_actions_ids_dialog(master, tk_master=None, title=None, info_text=None):
   d = WaitForActionsDialog(master, master if tk_master is None else tk_master, None, title, info_text)
   d.wait_window()
   if d.btn_OK:
      return True, d.out_selected_actions_ids
   else: 
      return False, None





