
from . import theme

# ******* messagebox types *********************
INFO = "Information"
WARNING = "Warning"
ERROR = "Error"
YESNO = "Question"
YESNOCANCEL = "Question with canceling"
OKCANCEL = "Confirmation"
CANCEL = "Processing"


class MessageBox(theme.MessageBox_Toplevel):

   def __init__(self, master, title=None, message="", type=INFO):
      super().__init__(master)
      self.last_grab = self.grab_current()
      self.grab_set()
      if title is None: self.title(type)
      else: self.title(title)
      self.resizable(False, False)
      self.attributes('-toolwindow', True)

      label = theme.MessageBox_Label(self, text=message)

      b = self.buttons = []
      add_b = lambda s, f: b.append(theme.MessageBox_Button(self, text=s, command=f, width=10))
      if type in (INFO, WARNING, ERROR): 
         add_b("OK", self.on_ok)
         self.buttons[0].focus()
         self.bind("<Return>", self.on_ok)
         self.bind("<Escape>", self.on_ok)
      elif type == YESNO:
         add_b("Yes", self.on_yes)
         add_b("No", self.on_no)
         self.buttons[1].focus()
         self.result = False
         self.bind("<Return>", self.on_yes)
         self.bind("<Escape>", self.on_no)
         self.bind("<Key>", self.on_char_key)
      elif type ==  YESNOCANCEL:
         add_b("Yes", self.on_yes)
         add_b("No", self.on_no)
         add_b("Cancel", self.on_cancel)
         self.buttons[2].focus()
         self.result = None
         self.bind("<Return>", self.on_yes)
         self.bind("<Escape>", self.on_cancel)
         self.bind("<Key>", self.on_char_key)
      elif type == OKCANCEL:
         add_b("OK", self.on_ok)
         add_b("Cancel", self.on_cancel)
         self.buttons[1].focus()
         self.result = False
         self.bind("<Return>", self.on_ok)
         self.bind("<Escape>", self.on_cancel)
      elif type == CANCEL:
         add_b("Cancel", self.on_cancel)
         self.buttons[0].focus()
         self.result = True
             
      label.grid(row=0, column=1, columnspan=len(b), sticky="W", padx=40, pady=40)
      for i, x in enumerate(b, 1):
         x.grid(row=1, column=i, padx=30, pady=20)

      master.update_idletasks()
      w, h = self.winfo_reqwidth(), self.winfo_reqheight()
      self.geometry("{:n}x{:n}+{:n}+{:n}".format(w, h, master.winfo_x()+master.winfo_width()//2-w//2, master.winfo_y()+master.winfo_height()//2-h//2))


   def destroy(self):
      if self.last_grab is not None: self.last_grab.grab_set()
      super().destroy()


   def on_ok(self, event=None):
      self.result = True
      self.destroy()
   
      
   def on_yes(self, event=None):
      self.result = True
      self.destroy()


   def on_no(self, event=None):
      self.result = False
      self.destroy()


   def on_cancel(self, event=None):
      self.result = None
      self.destroy()


   def on_char_key(self, event):
      if event.char == "n": self.on_no()
      elif event.char == "y": self.on_yes()
      

# *********** standard functions **************************

def msg_info(master, message, title=None):
   m = MessageBox(master, title=title, message=message, type=INFO)
   m.buttons[0].focus()
   m.wait_window()


def msg_warning(master, message, title=None):
   m = MessageBox(master, title=title, message=message, type=WARNING)
   m.wait_window()


def msg_error(master, message, title=None):
   m = MessageBox(master, title=title, message=message, type=ERROR)
   m.wait_window()
 

def msg_yesno(master, message, title=None):
   m = MessageBox(master, title=title, message=message, type=YESNO)
   m.wait_window()
   return m.result


def msg_yesnocancel(master, message, title=None):
   m = MessageBox(master, title=title, message=message, type=YESNOCANCEL)
   m.wait_window()
   return m.result


def msg_okcancel(master, message, title=None):
   m = MessageBox(master, title=title, message=message, type=OKCANCEL)
   m.wait_window()
   return m.result


def msg_cancel(master, message, title=None):
   m = MessageBox(master, title=title, message=message, type=CANCEL)
   return m