import tkinter as tk
import tkinter.ttk as ttk
import tkinter.scrolledtext as scrolledtext
import yaml


# ************* widgets classes **********************

class Action_Button(tk.Button):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Action_Button")
      super().__init__(*args, **newkwargs)
       
class Action_Frame(tk.Frame):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Action_Frame")
      super().__init__(*args, **newkwargs)


class Action_Label(tk.Label):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Action_Label")
      super().__init__(*args, **newkwargs)


class Action_ScrolledText(scrolledtext.ScrolledText):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Action_ScrolledText")
      super().__init__(*args, **newkwargs)


class Button(tk.Button):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Button")      
      super().__init__(*args, **newkwargs)


class Combobox(ttk.Combobox):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Combobox")
      t = {}
      a = ["bg", "fg", "selectcolor"]
      for s in a:
         if s in newkwargs: t[s] = newkwargs.pop(s)
      super().__init__(*args, **newkwargs)
   

class Checkbutton(tk.Checkbutton):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Checkbutton")
      super().__init__(*args, **newkwargs)
       

class Entry(tk.Entry):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Entry")
      super().__init__(*args, **newkwargs)
 
class Frame(tk.Frame):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Frame")
      super().__init__(*args, **newkwargs)


class Label(tk.Label):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Label")
      super().__init__(*args, **newkwargs)


class Listbox(tk.Listbox):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Listbox")
      super().__init__(*args, **newkwargs)


class Menu(tk.Menu):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Menu")
      super().__init__(*args, **newkwargs)


class MessageBox_Button(tk.Button):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "MessageBox_Button")
      super().__init__(*args, **newkwargs)


class MessageBox_Label(tk.Label):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "MessageBox_Label")
      super().__init__(*args, **newkwargs)


class MessageBox_Toplevel(tk.Toplevel):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "MessageBox_Toplevel")
      super().__init__(*args, **newkwargs)

class PanedWindow(tk.PanedWindow):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "PanedWindow")
      super().__init__(*args, **newkwargs)


class Protocol_ScrolledText(scrolledtext.ScrolledText):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Protocol_ScrolledText")
      super().__init__(*args, **newkwargs)


class Radiobutton(tk.Radiobutton):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Radiobutton")
      super().__init__(*args, **newkwargs)


class Scrollbar(tk.Scrollbar):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Scrollbar")
      super().__init__(*args, **newkwargs)

class ScrolledText(scrolledtext.ScrolledText):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "ScrolledText")
      super().__init__(*args, **newkwargs)

class StatusBar_Frame(tk.Frame):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "StatusBar_Frame")
      super().__init__(*args, **newkwargs)


class Text(tk.Text):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Text")
      super().__init__(*args, **newkwargs)


class ToolBar_Frame(tk.Frame):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "ToolBar_Frame")
      super().__init__(*args, **newkwargs)


class ToolBar_Button(tk.Button):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "ToolBar_Button")
      super().__init__(*args, **newkwargs)


class Toplevel(tk.Toplevel):
   def __init__(self, *args, **kwargs):
      newkwargs = _put_theme_params(kwargs, "Toplevel")
      super().__init__(*args, **newkwargs)

   def set_geometry(self, resizable=True):
      self.resizable(width=resizable, height=resizable)
      self.update()     
      h = self.winfo_reqheight()      
      w = self.winfo_reqwidth()
      self.geometry("{:n}x{:n}+{:n}+{:n}".format(w, h, self.master.winfo_x()+self.master.winfo_width()//2-w//2, self.master.winfo_y()+self.master.winfo_height()//2-h//2))
      self.minsize(w, h)
      
      




# ************ general execution ********************

class Theme:
   params = {}


def _put_theme_params(func_params, widget_type):
   if widget_type in Theme.params: 
      return func_params | {u:p for u, p in Theme.params[widget_type].items() if u not in func_params}
   else:
      return func_params


def load_theme(yaml_file_name):
   with open(yaml_file_name) as f:
      Theme.params = yaml.safe_load(f)
      
      
      
      




